@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2006,2007 Olly Betts
+ * Copyright 2003,2004,2006,2007,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -267,22 +267,15 @@ Xapian::Document::Internal::get_value(Xapian::valueno valueid) const
 	if (i == values.end()) return "";
 	return i->second;
     }
-    if (!database) return "";
+    if (!database.get()) return "";
     return do_get_value(valueid);
 }
 	
-map<Xapian::valueno, string>
-Xapian::Document::Internal::get_all_values() const
-{
-    need_values();
-    return values;
-}
-
 string
 Xapian::Document::Internal::get_data() const
 {
     if (data_here) return data;
-    if (!database) return "";
+    if (!database.get()) return "";
     return do_get_data();
 }
 
@@ -298,9 +291,9 @@ Xapian::Document::Internal::open_term_list() const
 {
     DEBUGCALL(MATCH, TermList *, "Document::Internal::open_term_list", "");
     if (terms_here) {
-	RETURN(new MapTermList(terms.begin(), terms.end(), terms.size()));
+	RETURN(new MapTermList(terms.begin(), terms.end()));
     }
-    if (!database) return NULL;
+    if (!database.get()) RETURN(NULL);
     RETURN(database->open_term_list(did));
 }
 
@@ -308,7 +301,13 @@ void
 Xapian::Document::Internal::add_value(Xapian::valueno valueno, const string &value)
 {
     need_values();
-    values[valueno] = value;
+    if (!value.empty()) {
+	values[valueno] = value;
+    } else {
+	// Empty values aren't stored, but replace any existing value by
+	// removing it.
+	values.erase(valueno);
+    }
     value_nos.clear();
 }
 
@@ -411,7 +410,7 @@ Xapian::Document::Internal::termlist_count() const
 {
     if (!terms_here) {
 	// How equivalent is this line to the rest?
-	// return database ? database->open_term_list(did)->get_approx_size() : 0;
+	// return database.get() ? database->open_term_list(did)->get_approx_size() : 0;
 	need_terms();
     }
     Assert(terms_here);
@@ -422,7 +421,7 @@ void
 Xapian::Document::Internal::need_terms() const
 {
     if (terms_here) return;
-    if (database) {
+    if (database.get()) {
 	Xapian::TermIterator t(database->open_term_list(did));
 	Xapian::TermIterator tend(NULL);
 	for ( ; t != tend; ++t) {
@@ -464,7 +463,7 @@ Xapian::Document::Internal::get_description() const
 	description += "terms[" + om_tostring(terms.size()) + "]";
     }
 
-    if (database) {
+    if (database.get()) {
 	if (data_here || values_here || terms_here) description += ", ";
 	description += "doc=";
 	description += "?"; // do_get_description(); ?
@@ -479,10 +478,16 @@ void
 Xapian::Document::Internal::need_values() const
 {
     if (!values_here) {
-	if (database) {
+	if (database.get()) {
 	    values = do_get_all_values();
 	    value_nos.clear();
 	}
 	values_here = true;
     }
 }
+
+Xapian::Document::Internal::~Internal()
+{
+    if (database.get())
+	database->invalidate_doc_object(this);
+}
