@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
  * Copyright 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -94,15 +94,13 @@ RSet::add_document(Xapian::docid did)
 void
 RSet::remove_document(Xapian::docid did)
 {
-    set<Xapian::docid>::iterator i = internal->items.find(did);
-    if (i != internal->items.end()) internal->items.erase(i);
+    internal->items.erase(did);
 }
 
 bool
 RSet::contains(Xapian::docid did) const
 {
-    set<Xapian::docid>::iterator i = internal->items.find(did);
-    return i != internal->items.end();
+    return internal->items.find(did) != internal->items.end();
 }
 
 string
@@ -118,11 +116,11 @@ RSet::Internal::get_description() const
 
     set<Xapian::docid>::const_iterator i;
     for (i = items.begin(); i != items.end(); ++i) {
-	if (!description.empty()) description += ", ";
+	if (i != items.begin()) description += ", ";
 	description += om_tostring(*i);
     }
 
-    description = "RSet(" + description + ")";
+    description += ')';
 
     return description;
 }
@@ -648,6 +646,10 @@ Enquire::Internal::get_mset(Xapian::doccount first, Xapian::doccount maxitems,
 	      maxitems << ", " << check_at_least << ", " << rset << ", " <<
 	      mdecider << ", " << matchspy);
 
+    if (percent_cutoff && (sort_by == VAL || sort_by == VAL_REL)) {
+	throw Xapian::UnimplementedError("Use of a percentage cutoff while sorting primary by value isn't currently supported");
+    }
+
     if (weight == 0) {
 	weight = new BM25Weight;
     }
