@@ -29,13 +29,8 @@
 #include "utils.h"
 
 #include <xapian/error.h>
-#include <xapian/enquire.h>
-
 #include <xapian/termiterator.h>
 
-#include <vector>
-#include <map>
-#include <algorithm>
 #include <math.h>
 
 namespace Xapian {
