@@ -24,34 +24,33 @@
 
 #include <config.h>
 
-#include <xapian/error.h>
-
-#include "safeerrno.h"
-
 #include "flint_database.h"
-#include "utils.h"
-#include "omdebug.h"
-#include "autoptr.h"
+
 #include <xapian/error.h>
 #include <xapian/valueiterator.h>
 
+#include "autoptr.h"
 #include "contiguousalldocspostlist.h"
-#include "flint_modifiedpostlist.h"
-#include "flint_postlist.h"
 #include "flint_alldocspostlist.h"
-#include "flint_termlist.h"
-#include "flint_positionlist.h"
-#include "flint_utils.h"
-#include "flint_record.h"
-#include "flint_values.h"
-#include "flint_document.h"
 #include "flint_alltermslist.h"
+#include "flint_document.h"
 #include "flint_lock.h"
+#include "flint_metadata.h"
+#include "flint_modifiedpostlist.h"
+#include "flint_positionlist.h"
+#include "flint_postlist.h"
+#include "flint_record.h"
 #include "flint_spellingwordslist.h"
+#include "flint_termlist.h"
+#include "flint_utils.h"
+#include "flint_values.h"
+#include "omdebug.h"
+#include "safeerrno.h"
+#include "safesysstat.h"
 #include "stringutils.h"
+#include "utils.h"
 
 #include <sys/types.h>
-#include "safesysstat.h"
 
 #include <list>
 #include <string>
@@ -388,6 +387,41 @@ FlintDatabase::get_database_write_lock()
     }
 }
 
+void
+FlintDatabase::modifications_failed(flint_revision_number_t old_revision,
+				    flint_revision_number_t new_revision,
+				    const string & msg)
+{
+    // Modifications failed.  Wipe all the modifications from memory.
+    try {
+	// Discard any buffered changes and reinitialised cached values
+	// from the table.
+	cancel();
+
+	// Reopen tables with old revision number.
+	open_tables(old_revision);
+
+	// Increase revision numbers to new revision number plus one,
+	// writing increased numbers to all tables.
+	++new_revision;
+	set_revision_number(new_revision);
+    } catch (const Xapian::Error &e) {
+	// Permanently close the table, since we can't get it into a
+	// consistent state, to avoid risk of database corruption.
+	postlist_table.close(true);
+	position_table.close(true);
+	termlist_table.close(true);
+	value_table.close(true);
+	synonym_table.close(true);
+	spelling_table.close(true);
+	record_table.close(true);
+	lock.release();
+	throw Xapian::DatabaseError("Modifications failed (" + msg +
+				    "), and cannot set consistent table "
+				    "revision numbers: " + e.get_msg());
+    }
+}
+
 void
 FlintDatabase::apply()
 {
@@ -407,23 +441,11 @@ FlintDatabase::apply()
 
     try {
 	set_revision_number(new_revision);
+    } catch (const Xapian::Error &e) {
+	modifications_failed(old_revision, new_revision, e.get_description());
+	throw;
     } catch (...) {
-	// Modifications failed.  Wipe all the modifications from memory.
-	try {
-	    // Discard any buffered changes and reinitialised cached values
-	    // from the table.
-	    cancel();
-
-	    // Reopen tables with old revision number.
-	    open_tables(old_revision);
-
-	    // Increase revision numbers to new revision number plus one,
-	    // writing increased numbers to all tables.
-	    ++new_revision;
-	    set_revision_number(new_revision);
-	} catch (const Xapian::Error &e) {
-	    throw Xapian::DatabaseError("Modifications failed, and cannot set consistent table revision numbers: " + e.get_msg());
-	}
+	modifications_failed(old_revision, new_revision, "Unknown error");
 	throw;
     }
 }
@@ -611,6 +633,16 @@ FlintDatabase::open_synonym_keylist(const string & prefix) const
 				    prefix);
 }
 
+TermList *
+FlintDatabase::open_metadata_keylist(const std::string &prefix) const
+{
+    DEBUGCALL(DB, string, "FlintDatabase::open_metadata_keylist", "");
+    FlintCursor * cursor = postlist_table.cursor_get();
+    if (!cursor) return NULL;
+    return new FlintMetadataTermList(Xapian::Internal::RefCntPtr<const FlintDatabase>(this),
+				     cursor, prefix);
+}
+
 string
 FlintDatabase::get_metadata(const string & key) const
 {
@@ -631,7 +663,9 @@ FlintWritableDatabase::FlintWritableDatabase(const string &dir, int action,
 	  doclens(),
 	  mod_plists(),
 	  change_count(0),
-	  flush_threshold(0)
+	  flush_threshold(0),
+	  modify_shortcut_document(NULL),
+	  modify_shortcut_docid(0)
 {
     DEBUGCALL(DB, void, "FlintWritableDatabase", dir << ", " << action << ", "
 	      << block_size);
@@ -701,9 +735,9 @@ FlintWritableDatabase::add_document_(Xapian::docid did,
 	{
 	    Xapian::ValueIterator value = document.values_begin();
 	    Xapian::ValueIterator value_end = document.values_end();
-	    for ( ; value != value_end; ++value) {
-		value_table.add_value(*value, did, value.get_valueno());
-	    }
+	    string s;
+	    value_table.encode_values(s, value, value_end);
+	    value_table.set_encoded_values(did, s);
 	}
 
 	flint_doclen_t new_doclen = 0;
@@ -786,6 +820,13 @@ FlintWritableDatabase::delete_document(Xapian::docid did)
     DEBUGCALL(DB, void, "FlintWritableDatabase::delete_document", did);
     Assert(did != 0);
 
+    if (rare(modify_shortcut_docid == did)) {
+	// The modify_shortcut document can't be used for a modification
+	// shortcut now, because it's been deleted!
+	modify_shortcut_document = NULL;
+	modify_shortcut_docid = 0;
+    }
+
     // Remove the record.  If this fails, just propagate the exception since
     // the state should still be consistent (most likely it's
     // DocNotFoundError).
@@ -872,73 +913,74 @@ FlintWritableDatabase::replace_document(Xapian::docid did,
 	    return;
 	}
 
-	// OK, now add entries to remove the postings in the underlying record.
-	Xapian::Internal::RefCntPtr<const FlintWritableDatabase> ptrtothis(this);
-	FlintTermList termlist(ptrtothis, did);
-
-	termlist.next();
-	while (!termlist.at_end()) {
-	    string tname = termlist.get_termname();
-	    termcount wdf = termlist.get_wdf();
-
-	    map<string, pair<termcount_diff, termcount_diff> >::iterator i;
-	    i = freq_deltas.find(tname);
-	    if (i == freq_deltas.end()) {
-		freq_deltas.insert(make_pair(tname, make_pair(-1, -termcount_diff(wdf))));
+	// Check for a document read from this database being replaced - ie, a
+	// modification operation.
+	bool modifying = false;
+	if (modify_shortcut_docid &&
+	    document.internal->get_docid() == modify_shortcut_docid) {
+	    if (document.internal.get() == modify_shortcut_document) {
+		// We have a docid, it matches, and the pointer matches, so we
+		// can skip modification of any data which hasn't been modified
+		// in the document.
+		modifying = true;
+		DEBUGLINE(DB, "Detected potential document modification shortcut.");
 	    } else {
-		--i->second.first;
-		i->second.second -= wdf;
+		// The modify_shortcut document can't be used for a
+		// modification shortcut now, because it's about to be
+		// modified.
+		modify_shortcut_document = NULL;
+		modify_shortcut_docid = 0;
 	    }
+	}
+  
+	if (!modifying || document.internal->terms_modified()) {
+	    // FIXME - in the case where there is overlap between the new
+	    // termlist and the old termlist, it would be better to compare the
+	    // two lists, and make the minimum set of modifications required.
+	    // This would lead to smaller changesets for replication, and
+	    // probably be faster overall.
 
-	    // Remove did from tname's postlist
-	    map<string, map<docid, pair<char, termcount> > >::iterator j;
-	    j = mod_plists.find(tname);
-	    if (j == mod_plists.end()) {
-		map<docid, pair<char, termcount> > m;
-		j = mod_plists.insert(make_pair(tname, m)).first;
-	    }
-
-	    map<docid, pair<char, termcount> >::iterator k;
-	    k = j->second.find(did);
-	    if (k == j->second.end()) {
-		j->second.insert(make_pair(did, make_pair('D', 0u)));
-	    } else {
-		// Modifying a document we added/modified since the last flush.
-		k->second = make_pair('D', 0u);
-	    }
+	    // First, add entries to remove the postings in the underlying record.
+	    Xapian::Internal::RefCntPtr<const FlintWritableDatabase> ptrtothis(this);
+	    FlintTermList termlist(ptrtothis, did);
 
 	    termlist.next();
-	}
+	    while (!termlist.at_end()) {
+		string tname = termlist.get_termname();
+		termcount wdf = termlist.get_wdf();
 
-	total_length -= termlist.get_doclength();
+		map<string, pair<termcount_diff, termcount_diff> >::iterator i;
+		i = freq_deltas.find(tname);
+		if (i == freq_deltas.end()) {
+		    freq_deltas.insert(make_pair(tname, make_pair(-1, -termcount_diff(wdf))));
+		} else {
+		    --i->second.first;
+		    i->second.second -= wdf;
+		}
 
-	// Replace the record
-	record_table.replace_record(document.get_data(), did);
+		// Remove did from tname's postlist
+		map<string, map<docid, pair<char, termcount> > >::iterator j;
+		j = mod_plists.find(tname);
+		if (j == mod_plists.end()) {
+		    map<docid, pair<char, termcount> > m;
+		    j = mod_plists.insert(make_pair(tname, m)).first;
+		}
 
-	// FIXME: we read the values delete them and then replace in case
-	// they come from where they're going!  Better to ask Document
-	// nicely and shortcut in this case!
-	{
-	    list<pair<string, Xapian::valueno> > tmp;
-	    Xapian::ValueIterator value = document.values_begin();
-	    Xapian::ValueIterator value_end = document.values_end();
-	    for ( ; value != value_end; ++value) {
-		tmp.push_back(make_pair(*value, value.get_valueno()));
+		map<docid, pair<char, termcount> >::iterator k;
+		k = j->second.find(did);
+		if (k == j->second.end()) {
+		    j->second.insert(make_pair(did, make_pair('D', 0u)));
+		} else {
+		    // Modifying a document we added/modified since the last flush.
+		    k->second = make_pair('D', 0u);
+		}
+
+		termlist.next();
 	    }
-//	    value_table.add_value(*value, did, value.get_valueno());
 
-	    // Replace the values.
-	    value_table.delete_all_values(did);
+	    total_length -= termlist.get_doclength();
 
-	    // Set the values.
-	    list<pair<string, Xapian::valueno> >::const_iterator i;
-	    for (i = tmp.begin(); i != tmp.end(); ++i) {
-		value_table.add_value(i->first, did, i->second);
-	    }
-	}
-
-	flint_doclen_t new_doclen = 0;
-	{
+	    flint_doclen_t new_doclen = 0;
 	    Xapian::TermIterator term = document.termlist_begin();
 	    Xapian::TermIterator term_end = document.termlist_end();
 	    for ( ; term != term_end; ++term) {
@@ -978,21 +1020,39 @@ FlintWritableDatabase::replace_document(Xapian::docid did,
 		PositionIterator it = term.positionlist_begin();
 		PositionIterator it_end = term.positionlist_end();
 		if (it != it_end) {
-		    position_table.set_positionlist(
-			did, tname, it, it_end);
+		    position_table.set_positionlist(did, tname, it, it_end);
 		} else {
 		    position_table.delete_positionlist(did, tname);
 		}
 	    }
+	    DEBUGLINE(DB, "Calculated doclen for replacement document " << did << " as " << new_doclen);
+
+	    // Set the termlist
+	    termlist_table.set_termlist(did, document, new_doclen);
+
+	    // Set the new document length
+	    doclens[did] = new_doclen;
+	    total_length += new_doclen;
 	}
-	DEBUGLINE(DB, "Calculated doclen for replacement document " << did << " as " << new_doclen);
 
-	// Set the termlist
-	termlist_table.set_termlist(did, document, new_doclen);
+	if (!modifying || document.internal->data_modified()) {
+	    // Replace the record
+	    record_table.replace_record(document.get_data(), did);
+	}
 
-	// Set the new document length
-	doclens[did] = new_doclen;
-	total_length += new_doclen;
+	if (!modifying || document.internal->values_modified()) {
+	    // FIXME: we read the values delete them and then replace in case
+	    // they come from where they're going!  Better to ask Document
+	    // nicely and shortcut in this case!
+	    Xapian::ValueIterator value = document.values_begin();
+	    Xapian::ValueIterator value_end = document.values_end();
+	    string s;
+	    value_table.encode_values(s, value, value_end);
+
+	    // Replace the values.
+	    value_table.delete_all_values(did);
+	    value_table.set_encoded_values(did, s);
+	}
     } catch (const Xapian::DocNotFoundError &) {
 	(void)add_document_(did, document);
 	return;
@@ -1011,6 +1071,18 @@ FlintWritableDatabase::replace_document(Xapian::docid did,
     }
 }
 
+Xapian::Document::Internal *
+FlintWritableDatabase::open_document(Xapian::docid did, bool lazy) const
+{
+    DEBUGCALL(DB, Xapian::Document::Internal *, "FlintWritableDatabase::open_document",
+	      did << ", " << lazy);
+    modify_shortcut_document = FlintDatabase::open_document(did, lazy);
+    // Store the docid only after open_document() successfully returns, so an
+    // attempt to open a missing document doesn't overwrite this.
+    modify_shortcut_docid = did;
+    RETURN(modify_shortcut_document);
+}
+
 Xapian::doclength
 FlintWritableDatabase::get_doclength(Xapian::docid did) const
 {
@@ -1159,3 +1231,12 @@ FlintWritableDatabase::set_metadata(const string & key, const string & value)
 	postlist_table.add(btree_key, value);
     }
 }
+
+void
+FlintWritableDatabase::invalidate_doc_object(Xapian::Document::Internal * obj) const
+{
+    if (obj == modify_shortcut_document) {
+	modify_shortcut_document = NULL;
+	modify_shortcut_docid = 0;
+    }
+}
