@@ -177,6 +177,15 @@ class FlintDatabase : public Xapian::Database::Internal {
 	 */
 	void reopen();
 
+	/** Called if a modifications fail.
+	 *
+	 *  @param msg is a string description of the exception that was
+	 *  raised when the modifications failed.
+	 */
+	void modifications_failed(flint_revision_number_t old_revision,
+				  flint_revision_number_t new_revision,
+				  const std::string & msg);
+
 	/** Apply any outstanding changes to the tables.
 	 *
 	 *  If an error occurs during this operation, this will be signalled
@@ -228,7 +237,7 @@ class FlintDatabase : public Xapian::Database::Internal {
 	bool has_positions() const;
 
 	LeafPostList * open_post_list(const string & tname) const;
-	Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy = false) const;
+	Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy) const;
 
 	PositionList * open_position_list(Xapian::docid did, const string & term) const;
 	TermList * open_term_list(Xapian::docid did) const;
@@ -242,6 +251,7 @@ class FlintDatabase : public Xapian::Database::Internal {
 	TermList * open_synonym_keylist(const string & prefix) const;
 
 	string get_metadata(const string & key) const;
+	TermList * open_metadata_keylist(const std::string &prefix) const;
 	//@}
 };
 
@@ -267,6 +277,18 @@ class FlintWritableDatabase : public FlintDatabase {
 	/// If change_count reaches this threshold we automatically flush.
 	Xapian::doccount flush_threshold;
 
+	/** A pointer to the last document which was returned by
+	 *  open_document(), or NULL if there is no such valid document.  This
+	 *  is used purely for comparing with a supplied document to help with
+	 *  optimising replace_document.  When the document internals are
+	 *  deleted, this pointer gets set to NULL.
+	 */
+	mutable Xapian::Document::Internal * modify_shortcut_document;
+
+	/** The document ID for the last document returned by open_document().
+	 */
+	mutable Xapian::docid modify_shortcut_docid;
+
 	/// Flush any unflushed postlist changes, but don't commit them.
 	void flush_postlist_changes() const;
 
@@ -292,6 +314,10 @@ class FlintWritableDatabase : public FlintDatabase {
 #endif
 	void delete_document(Xapian::docid did);
 	void replace_document(Xapian::docid did, const Xapian::Document & document);
+
+	Xapian::Document::Internal * open_document(Xapian::docid did,
+						   bool lazy = false) const;
+
 	//@}
 
     public:
@@ -330,6 +356,7 @@ class FlintWritableDatabase : public FlintDatabase {
 	void clear_synonyms(const string & word) const;
 
 	void set_metadata(const string & key, const string & value);
+	void invalidate_doc_object(Xapian::Document::Internal * obj) const;
 	//@}
 };
 
