@@ -1,7 +1,7 @@
 /** @file flint_spelling.h
  * @brief Spelling correction data for a flint database.
  */
-/* Copyright (C) 2007 Olly Betts
+/* Copyright (C) 2007,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,8 +53,7 @@ inline bool operator<(const fragment &a, const fragment &b) {
 }
 
 class FlintSpellingTable : public FlintTable {
-    void add_fragment(fragment frag, const string & word);
-    void remove_fragment(fragment frag, const string & word);
+    void toggle_fragment(fragment frag, const string & word);
 
     std::map<std::string, Xapian::termcount> wordfreq_changes;
     std::map<fragment, std::set<std::string> > termlist_deltas;
@@ -68,7 +67,7 @@ class FlintSpellingTable : public FlintTable {
      *  @param dbdir		The directory the flint database is stored in.
      *  @param readonly		true if we're opening read-only, else false.
      */
-    FlintSpellingTable(std::string dbdir, bool readonly)
+    FlintSpellingTable(const std::string & dbdir, bool readonly)
 	: FlintTable(dbdir + "/spelling.", readonly, Z_DEFAULT_STRATEGY, true) { }
 
     // Merge in batched-up changes.
@@ -137,9 +136,6 @@ class FlintSpellingTermList : public TermList {
     FlintSpellingTermList(const std::string & data_)
 	: data(data_), p(0) { }
 
-    /// Destructor.
-    ~FlintSpellingTermList();
-
     Xapian::termcount get_approx_size() const;
 
     std::string get_termname() const;
