@@ -1,7 +1,7 @@
 /* flint_table.h: Btree implementation
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -281,7 +281,7 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 *  @param lazy		If true, don't create the table until it's
 	 *			needed.
 	 */
-	FlintTable(string path_, bool readonly_,
+	FlintTable(const string & path_, bool readonly_,
 		   int compress_strategy_ = DONT_COMPRESS, bool lazy = false);
 
 	/** Close the Btree.
@@ -293,8 +293,10 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 
 	/** Close the Btree.  This closes and frees any of the btree
 	 *  structures which have been created and opened.
+	 *
+	 *  @param permanent If true, the Btree will not reopen on demand.
 	 */
-	void close();
+	void close(bool permanent=false);
 
 	/** Determine whether the btree exists on disk.
 	 */
@@ -581,6 +583,16 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	void split_root(uint4 split_n);
 	void form_key(const string & key) const;
 
+	char other_base_letter() const {
+	   return (base_letter == 'A') ? 'B' : 'A';
+	}
+
+	/// Allocate the zstream for deflating, if not already allocated.
+	void lazy_alloc_deflate_zstream() const;
+
+	/// Allocate the zstream for inflating, if not already allocated.
+	void lazy_alloc_inflate_zstream() const;
+
 	/** revision number of the opened B-tree. */
 	flint_revision_number_t revision_number;
 
@@ -602,7 +614,7 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	mutable bool both_bases;
 
 	/** the value 'A' or 'B' of the current base */
-	int base_letter;
+	char base_letter;
 
 	/** true if the root block is faked (not written to disk).
 	 * false otherwise.  This is true when the btree hasn't been
@@ -633,9 +645,6 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	/// For writing back as file baseA or baseB.
 	FlintTable_base base;
 
-	/// The base letter ('B' or 'A') of the next base.
-	char other_base_letter;
-
 	/// The path name of the B tree.
 	string name;
 
@@ -702,6 +711,12 @@ class XAPIAN_VISIBILITY_DEFAULT FlintTable {
 	 *  Z_RLE. */
 	int compress_strategy;
 
+	/// Zlib state object for deflating
+	mutable z_stream *deflate_zstream;
+
+	/// Zlib state object for inflating
+	mutable z_stream *inflate_zstream;
+
 	/// If true, don't create the table until it's needed.
 	bool lazy;
 
