@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2006 Olly Betts
+ * Copyright 2002,2003,2004,2006,2009 Olly Betts
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -319,13 +319,13 @@ unpack_string(const char ** src,
 	return false;
     }
 
-    result = string(*src, length);
+    result.assign(*src, length);
     *src += length;
     return true;
 }
 
 inline string
-pack_string(string value)
+pack_string(const string & value)
 {
     return pack_uint(value.size()) + value;
 }
