@@ -55,9 +55,6 @@ class InMemoryAllTermsList : public AllTermsList
 	/// Standard destructor for base class.
 	~InMemoryAllTermsList();
 
-        // Gets size of termlist
-	Xapian::termcount get_approx_size() const;
-
 	// Gets current termname
 	string get_termname() const;
 
