@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
  * Copyright 2006 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
@@ -78,10 +78,10 @@ InMemoryDoc::add_posting(const InMemoryTermEntry & post)
 //////////////
 
 InMemoryPostList::InMemoryPostList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db_,
-				   const InMemoryTerm & term)
-	: pos(term.docs.begin()),
-	  end(term.docs.end()),
-	  termfreq(term.term_freq),
+				   const InMemoryTerm & imterm)
+	: pos(imterm.docs.begin()),
+	  end(imterm.docs.end()),
+	  termfreq(imterm.term_freq),
 	  started(false),
 	  db(db_)
 {
@@ -146,7 +146,7 @@ InMemoryPostList::at_end() const
 string
 InMemoryPostList::get_description() const
 {
-    return "InMemoryPostList" + om_tostring(termfreq);
+    return "InMemoryPostList " + om_tostring(termfreq);
 }
 
 Xapian::doclength
@@ -181,13 +181,12 @@ InMemoryPostList::get_wdf() const
 InMemoryTermList::InMemoryTermList(Xapian::Internal::RefCntPtr<const InMemoryDatabase> db_,
 				   Xapian::docid did_,
 				   const InMemoryDoc & doc,
-				   Xapian::doclength len)
+				   Xapian::termcount len)
 	: pos(doc.terms.begin()), end(doc.terms.end()), terms(doc.terms.size()),
-	  started(false), db(db_), did(did_)
+	  started(false), db(db_), did(did_), document_length(len)
 {
     DEBUGLINE(DB, "InMemoryTermList::InMemoryTermList(): " <<
 	          terms << " terms starting from " << pos->tname);
-    document_length = len;
 }
 
 Xapian::termcount
@@ -342,7 +341,7 @@ InMemoryAllDocsPostList::at_end() const
 string
 InMemoryAllDocsPostList::get_description() const
 {
-    return "InMemoryAllDocsPostList" + om_tostring(did);
+    return "InMemoryAllDocsPostList " + om_tostring(did);
 }
 
 ///////////////////////////
@@ -365,8 +364,6 @@ LeafPostList *
 InMemoryDatabase::open_post_list(const string & tname) const
 {
     if (tname.empty()) {
-	if (termlists.empty())
-	    return new EmptyPostList();
 	Xapian::Internal::RefCntPtr<const InMemoryDatabase> ptrtothis(this);
 	return new InMemoryAllDocsPostList(ptrtothis);
     }
@@ -441,7 +438,7 @@ InMemoryDatabase::open_term_list(Xapian::docid did) const
 				 string(" not found"));
     }
     return new InMemoryTermList(Xapian::Internal::RefCntPtr<const InMemoryDatabase>(this), did,
-				termlists[did - 1], get_doclength(did));
+				termlists[did - 1], doclengths[did - 1]);
 }
 
 Xapian::Document::Internal *
@@ -466,6 +463,14 @@ InMemoryDatabase::get_metadata(const std::string & key) const
 	return string();
     return i->second;
 }
+ 
+TermList *
+InMemoryDatabase::open_metadata_keylist(const string &) const
+{
+    if (metadata.empty()) return NULL;
+    // FIXME: nobody implemented this yet...
+    throw Xapian::UnimplementedError("InMemory backend doesn't currently implement Database::metadata_keys_begin()");
+}
 
 void
 InMemoryDatabase::set_metadata(const std::string & key,
@@ -641,7 +646,7 @@ InMemoryDatabase::finish_add_doc(Xapian::docid did, const Xapian::Document &docu
 	Xapian::ValueIterator k_end = document.values_end();
 	for ( ; k != k_end; ++k) {
 	    values.insert(make_pair(k.get_valueno(), *k));
-	    DEBUGLINE(DB, "InMemoryDatabase::add_document(): adding value "
+	    DEBUGLINE(DB, "InMemoryDatabase::finish_add_doc(): adding value "
 		      << k.get_valueno() << " -> " << *k);
 	}
 	add_values(did, values);
@@ -653,7 +658,7 @@ InMemoryDatabase::finish_add_doc(Xapian::docid did, const Xapian::Document &docu
     for ( ; i != i_end; ++i) {
 	make_term(*i);
 
-	DEBUGLINE(DB, "InMemoryDatabase::add_document(): adding term "
+	DEBUGLINE(DB, "InMemoryDatabase::finish_add_doc(): adding term "
 		  << *i);
 	Xapian::PositionIterator j = i.positionlist_begin();
 	Xapian::PositionIterator j_end = i.positionlist_end();
