@@ -22,14 +22,19 @@
 #ifndef XAPIAN_INCLUDED_REMOTESERVER_H
 #define XAPIAN_INCLUDED_REMOTESERVER_H
 
-#include <xapian/database.h>
-#include <xapian/visibility.h>
+#include "xapian/database.h"
+#include "xapian/enquire.h"
+#include "xapian/visibility.h"
 
-#include "omassert.h"
 #include "remoteconnection.h"
-#include "stats.h"
 
-class NetworkStatsGatherer;
+#include <map>
+#include <string>
+
+// Forward declaration
+namespace Xapian { class Weight; }
+
+using namespace std;
 
 /** Remote backend server base class. */
 class XAPIAN_VISIBILITY_DEFAULT RemoteServer : private RemoteConnection {
@@ -62,9 +67,6 @@ class XAPIAN_VISIBILITY_DEFAULT RemoteServer : private RemoteConnection {
      */
     Xapian::timeout idle_timeout;
 
-    /// Global stats passed from the client and used in the match.
-    Stats global_stats;
-
     /// Registered weighting schemes.
     map<string, Xapian::Weight *> wtschemes;
 
@@ -160,9 +162,6 @@ class XAPIAN_VISIBILITY_DEFAULT RemoteServer : private RemoteConnection {
     /// Destructor.
     ~RemoteServer();
 
-    /// Return global Stats object.
-    const Stats & get_global_stats() { return global_stats; }
-
     /** Repeatedly accept messages from the client and process them.
      *
      *  The loop continues until either the connection is closed, or a
