@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -165,6 +165,7 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 	Xapian::doccount get_matches_upper_bound() const;
 
 	/** The maximum possible weight in the MSet.
+	 *
 	 *  This weight is likely not to be attained in the set of results,
 	 *  but represents an upper bound on the weight which a document
 	 *  could attain for the given query.
@@ -174,8 +175,8 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 	/** The greatest weight which is attained by any document in the
 	 *  database.
 	 *
-	 *  If firstitem == 0, this is the weight of the first entry in
-	 *  items.
+	 *  If firstitem == 0 and the primary ordering is by relevance, this is
+	 *  the weight of the first entry in the MSet. 
 	 *
 	 *  If no documents are found by the query, this will be 0.
 	 *
@@ -353,8 +354,18 @@ class XAPIAN_VISIBILITY_DEFAULT MSetIterator {
 
 	/** This returns the weight of the document as a percentage score.
 	 *
-	 *  The return value will be in the range 0 to 100:  0 meaning
-	 *  that the item did not match the query at all.
+	 *  The return value will be an integer in the range 0 to 100:  0
+	 *  meaning that the item did not match the query at all.
+	 *
+	 *  The intention is that the highest weighted document will get 100
+	 *  if it matches all the weight-contributing terms in the query.
+	 *  However, currently it may get a lower percentage score if you
+	 *  use a MatchDecider and the sorting is primarily by value.
+	 *  In this case, the percentage for a particular document may vary
+	 *  depending on the first, max_size, and checkatleast parameters
+	 *  passed to Enquire::get_mset() (this bug is hard to fix without
+	 *  having to apply the MatchDecider to potentially many more
+	 *  documents, which is potentially costly).
 	 */
 	Xapian::percent get_percent() const;
 
@@ -371,11 +382,13 @@ class XAPIAN_VISIBILITY_DEFAULT MSetIterator {
 	//@}
 };
 
+/// Equality test for MSetIterator objects.
 inline bool operator==(const MSetIterator &a, const MSetIterator &b)
 {
     return (a.index == b.index);
 }
 
+/// Inequality test for MSetIterator objects.
 inline bool operator!=(const MSetIterator &a, const MSetIterator &b)
 {
     return (a.index != b.index);
@@ -517,11 +530,13 @@ class XAPIAN_VISIBILITY_DEFAULT ESetIterator {
 	//@}
 };
 
+/// Equality test for ESetIterator objects.
 inline bool operator==(const ESetIterator &a, const ESetIterator &b)
 {
     return (a.index == b.index);
 }
 
+/// Inequality test for ESetIterator objects.
 inline bool operator!=(const ESetIterator &a, const ESetIterator &b)
 {
     return (a.index != b.index);
@@ -757,61 +772,70 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 
 	/** Set the sorting to be by value only.
 	 *
-	 *  NB sorting of values uses a string comparison, so you'll need to
-	 *  store numbers padded with leading zeros or spaces, or with the
-	 *  number of digits prepended.
+	 *  Note that sorting by values uses a string comparison, so to use
+	 *  this to sort by a numeric value you'll need to store the numeric
+	 *  values in a manner which sorts appropriately.  For example, you
+	 *  could use Xapian::sortable_serialise() (which works for floating
+	 *  point numbers as well as integers), or store numbers padded with
+	 *  leading zeros or spaces, or with the number of digits prepended.
 	 *
 	 * @param sort_key  value number to sort on.
 	 *
-	 * @param ascending  If true, documents values which sort higher by
-	 *		 string compare are better.  If false, the sort order
-	 *		 is reversed.  (default true)
+	 * @param reverse   If true, reverses the sort order.  (default true, but
+	 *		    this default is confusing and deprecated in 1.1.0, so
+	 *		    we recommend specifying this parameter explicitly).
          */
-	void set_sort_by_value(Xapian::valueno sort_key, bool ascending = true);
+	void set_sort_by_value(Xapian::valueno sort_key, bool reverse = true);
 
 	/** Set the sorting to be by key generated from values only.
 	 *
 	 * @param sorter    The functor to use for generating keys.
 	 *
-	 * @param ascending  If true, documents values which sort higher by
-	 *		 string compare are better.  If false, the sort order
-	 *		 is reversed.  (default true)
+	 * @param reverse   If true, reverses the sort order.  (default true, but
+	 *		    this default is confusing and deprecated in 1.1.0, so
+	 *		    we recommend specifying this parameter explicitly).
          */
-	void set_sort_by_key(Xapian::Sorter * sorter, bool ascending = true);
+	void set_sort_by_key(Xapian::Sorter * sorter, bool reverse = true);
 
 	/** Set the sorting to be by value, then by relevance for documents
 	 *  with the same value.
 	 *
-	 *  NB sorting of values uses a string comparison, so you'll need to
-	 *  store numbers padded with leading zeros or spaces, or with the
-	 *  number of digits prepended.
+	 *  Note that sorting by values uses a string comparison, so to use
+	 *  this to sort by a numeric value you'll need to store the numeric
+	 *  values in a manner which sorts appropriately.  For example, you
+	 *  could use Xapian::sortable_serialise() (which works for floating
+	 *  point numbers as well as integers), or store numbers padded with
+	 *  leading zeros or spaces, or with the number of digits prepended.
 	 *
 	 * @param sort_key  value number to sort on.
 	 *
-	 * @param ascending  If true, documents values which sort higher by
-	 *		 string compare are better.  If false, the sort order
-	 *		 is reversed.  (default true)
+	 * @param reverse   If true, reverses the sort order.  (default true, but
+	 *		    this default is confusing and deprecated in 1.1.0, so
+	 *		    we recommend specifying this parameter explicitly).
 	 */
 	void set_sort_by_value_then_relevance(Xapian::valueno sort_key,
-					      bool ascending = true);
+					      bool reverse = true);
 
 	/** Set the sorting to be by keys generated from values, then by
 	 *  relevance for documents with identical keys.
 	 *
 	 * @param sorter    The functor to use for generating keys.
 	 *
-	 * @param ascending  If true, keys which sort higher by
-	 *		 string compare are better.  If false, the sort order
-	 *		 is reversed.  (default true)
+	 * @param reverse   If true, reverses the sort order.  (default true, but
+	 *		    this default is confusing and deprecated in 1.1.0, so
+	 *		    we recommend specifying this parameter explicitly).
 	 */
 	void set_sort_by_key_then_relevance(Xapian::Sorter * sorter,
-					    bool ascending = true);
+					    bool reverse = true);
 
 	/** Set the sorting to be by relevance then value.
 	 *
-	 *  NB sorting of values uses a string comparison, so you'll need to
-	 *  store numbers padded with leading zeros or spaces, or with the
-	 *  number of digits prepended.
+	 *  Note that sorting by values uses a string comparison, so to use
+	 *  this to sort by a numeric value you'll need to store the numeric
+	 *  values in a manner which sorts appropriately.  For example, you
+	 *  could use Xapian::sortable_serialise() (which works for floating
+	 *  point numbers as well as integers), or store numbers padded with
+	 *  leading zeros or spaces, or with the number of digits prepended.
 	 *
 	 *  Note that with the default BM25 weighting scheme parameters,
 	 *  non-identical documents will rarely have the same weight, so
@@ -822,12 +846,12 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *
 	 * @param sort_key  value number to sort on.
 	 *
-	 * @param ascending  If true, documents values which sort higher by
-	 *		 string compare are better.  If false, the sort order
-	 *		 is reversed.  (default true)
+	 * @param reverse   If true, reverses the sort order.  (default true, but
+	 *		    this default is confusing and deprecated in 1.1.0, so
+	 *		    we recommend specifying this parameter explicitly).
 	 */
 	void set_sort_by_relevance_then_value(Xapian::valueno sort_key,
-					      bool ascending = true);
+					      bool reverse = true);
 
 	/** Set the sorting to be by relevance, then by keys generated from
 	 *  values.
@@ -841,12 +865,12 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *
 	 * @param sorter    The functor to use for generating keys.
 	 *
-	 * @param ascending  If true, keys which sort higher by
-	 *		 string compare are better.  If false, the sort order
-	 *		 is reversed.  (default true)
+	 * @param reverse   If true, reverses the sort order.  (default true, but
+	 *		    this default is confusing and deprecated in 1.1.0, so
+	 *		    we recommend specifying this parameter explicitly).
 	 */
 	void set_sort_by_relevance_then_key(Xapian::Sorter * sorter,
-					    bool ascending = true);
+					    bool reverse = true);
 
 	/** Get (a portion of) the match set for the current query.
 	 *
@@ -856,7 +880,19 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *		     A value of 10 corresponds to the first 10 items
 	 *		     being ignored, and the returned items starting
 	 *		     at the eleventh.
-	 *  @param maxitems  the maximum number of items to return.
+	 *  @param maxitems  the maximum number of items to return.  If you
+	 *		     want all matches, then you can pass the result
+	 *		     of calling get_doccount() on the Database object
+	 *		     (though if you are doing this so you can filter
+	 *		     results, you are likely to get much better
+	 *		     performance by using Xapian's match-time filtering
+	 *		     features instead).  You can pass 0 for maxitems
+	 *		     which will give you an empty MSet with valid
+	 *		     statistics (such as get_matches_estimated())
+	 *		     calculated without looking at any postings, which
+	 *		     is very quick, but means the estimates may be
+	 *		     more approximate and the bounds may be much
+	 *		     looser.
 	 *  @param checkatleast  the minimum number of items to check.  Because
 	 *		     the matcher optimises, it won't consider every
 	 *		     document which might match, so the total number
