@@ -1,7 +1,7 @@
 /** \file errorhandler.h
  *  \brief Decide if a Xapian::Error exception should be ignored.
  */
-/* Copyright (C) 2003,2006 Olly Betts
+/* Copyright (C) 2003,2006,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,11 @@
 #ifndef XAPIAN_INCLUDED_ERRORHANDLER_H
 #define XAPIAN_INCLUDED_ERRORHANDLER_H
 
-#include <xapian/error.h>
+#include <xapian/visibility.h>
+
+namespace Xapian {
+
+class Error;
 
 /** Decide if a Xapian::Error exception should be ignored.
  *
@@ -36,7 +40,7 @@
  *  also so that such errors can be logged and dead servers temporarily removed
  *  from use).
  */
-class Xapian::ErrorHandler {
+class XAPIAN_VISIBILITY_DEFAULT ErrorHandler {
     /// Don't allow assignment.
     void operator=(const ErrorHandler &);
 
@@ -69,7 +73,7 @@ class Xapian::ErrorHandler {
     ErrorHandler() {}
 
     /// We require a virtual destructor because we have virtual methods.
-    virtual ~ErrorHandler() {}
+    virtual ~ErrorHandler();
 
     /** Handle a Xapian::Error object.
      *
@@ -84,4 +88,6 @@ class Xapian::ErrorHandler {
     void operator()(Xapian::Error &error);
 };
 
+}
+
 #endif /* XAPIAN_INCLUDED_ERRORHANDLER_H */
