@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2003,2004,2005,2006,2007,2009 Olly Betts
  * Copyright 2006,2007,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -77,6 +77,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 
 	    /** Find occurrences of a list of terms with all the terms
 	     *  occurring within a specified window of positions.
+	     *
 	     *  Each occurrence of a term must be at a different position,
 	     *  but the order they appear in is irrelevant.
 	     *
@@ -87,8 +88,9 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 
 	    /** Find occurrences of a list of terms with all the terms
 	     *  occurring within a specified window of positions, and all
-	     *  the terms appearing in the order specified.  Each occurrence
-	     *  of a term must be at a different position.
+	     *  the terms appearing in the order specified.
+	     *
+	     *  Each occurrence of a term must be at a different position.
 	     *
 	     *  The window parameter should be specified for this operation,
 	     *  but will default to the number of terms in the list.
@@ -173,7 +175,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	 *
 	 *  @deprecated This method is deprecated because it isn't useful,
 	 *  since none of the current query operators can be usefully
-	 *  applied to a single subquery with a parameter value.
+	 *  applied to a single subquery without a parameter value.
 	 */
 	XAPIAN_DEPRECATED(Query(Query::op op_, Xapian::Query q));
 
@@ -323,15 +325,6 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 	/// Within query frequency of this term - leaf node only
 	Xapian::termcount wqf;
 
-	/** swap the contents of this with another Xapian::Query::Internal,
-	 *  in a way which is guaranteed not to throw.  This is
-	 *  used with the assignment operator to make it exception
-	 *  safe.
-	 *  It's important to adjust swap with any addition of
-	 *  member variables!
-	 */
-	void swap(Query::Internal &other);
-
 	/// Copy another Xapian::Query::Internal into self.
 	void initialise_from_copy(const Query::Internal & copyme);
 
@@ -369,7 +362,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 	/** Flatten a query structure, by changing, for example,
 	 *  "A NEAR (B AND C)" to "(A NEAR B) AND (A NEAR C)"
 	 */
-	void flatten_subqs();
+	Xapian::Query::Internal * flatten_subqs();
 
         /** Implementation of serialisation; called recursively.
          */
@@ -402,10 +395,15 @@ class XAPIAN_VISIBILITY_DEFAULT Query::Internal : public Xapian::Internal::RefCn
 
 	static Xapian::Query::Internal * unserialise(const std::string &s);
 
-	/** Add a subquery.
-	 */
+	/** Add a subquery. */
 	void add_subquery(const Query::Internal * subq);
 
+	/** Add a subquery without copying it.
+	 *
+	 *  subq is owned by the object this is called on after the call.
+	 */
+	void add_subquery_nocopy(Query::Internal * subq);
+
 	void set_dbl_parameter(double dbl_parameter_);
 
 	double get_dbl_parameter() const;
