@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
  * Copyright 2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -44,7 +44,7 @@ LocalSubMatch::LocalSubMatch(const Xapian::Database::Internal *db_,
 	const Xapian::RSet & omrset,
 	const Xapian::Weight *wt_factory_)
 	: orig_query(*query), qlen(qlen_), db(db_),
-	  rset(db, omrset), wt_factory(wt_factory_)
+	  rset(db, omrset), wt_factory(wt_factory_), term_info(NULL)
 {
     DEBUGCALL(MATCH, void, "LocalSubMatch::LocalSubMatch",
 	      db << ", " << query << ", " << qlen_ << ", " << omrset << ", " <<
@@ -65,11 +65,8 @@ LocalSubMatch::prepare_match(bool /*nowait*/, Stats & total_stats)
     Xapian::TermIterator titer = orig_query.get_terms();
     Xapian::TermIterator terms_end(NULL);
     for ( ; titer != terms_end; ++titer) {
-	if ((*titer).empty()) {
-	    my_stats.set_termfreq(*titer, db->get_doccount());
-	} else {
-	    my_stats.set_termfreq(*titer, db->get_termfreq(*titer));
-	}
+	Assert(!(*titer).empty());
+	my_stats.set_termfreq(*titer, db->get_termfreq(*titer));
 	rset.will_want_reltermfreq(*titer);
     }
     rset.contribute_stats(my_stats);
@@ -93,13 +90,13 @@ LocalSubMatch::get_postlist_and_term_info(MultiMatch * matcher,
 {
     DEBUGCALL(MATCH, PostList *, "LocalSubMatch::get_postlist_and_term_info",
 	      matcher << ", [termfreqandwts]");
+    term_info = termfreqandwts;
 
     // Build the postlist tree for the query.  This calls
     // LocalSubMatch::postlist_from_op_leaf_query() for each term in the query,
     // which builds term_info as a side effect.
     QueryOptimiser opt(*db, *this, matcher);
     PostList * pl = opt.optimise_query(&orig_query);
-    if (termfreqandwts) *termfreqandwts = term_info;
 
     // We only need an ExtraWeightPostList if there's an extra weight
     // contribution.
@@ -136,15 +133,23 @@ LocalSubMatch::postlist_from_op_leaf_query(const Xapian::Query::Internal *query,
 	}
     }
 
-    map<string, Xapian::MSet::Internal::TermFreqAndWeight>::iterator i;
-    i = term_info.find(query->tname);
-    if (i == term_info.end()) {
-	Xapian::doccount tf = stats->get_termfreq(query->tname);
-	Xapian::weight weight = boolean ? 0 : wt->get_maxpart();
-	Xapian::MSet::Internal::TermFreqAndWeight info(tf, weight);
-	term_info.insert(make_pair(query->tname, info));
-    } else if (!boolean) {
-	i->second.termweight += wt->get_maxpart();
+    if (term_info) {
+	map<string, Xapian::MSet::Internal::TermFreqAndWeight>::iterator i;
+	i = term_info->find(query->tname);
+	if (i == term_info->end()) {
+	    Xapian::doccount tf = stats->get_termfreq(query->tname);
+	    Xapian::weight weight = boolean ? 0 : wt->get_maxpart();
+	    Xapian::MSet::Internal::TermFreqAndWeight info(tf, weight);
+	    DEBUGLINE(MATCH, "Setting term_info[" << query->tname << "] "
+		      "to (" << tf << ", " << weight << ")");
+	    term_info->insert(make_pair(query->tname, info));
+	} else if (!boolean) {
+	    i->second.termweight += wt->get_maxpart();
+	    AssertEq(stats->get_termfreq(query->tname), i->second.termfreq);
+	    DEBUGLINE(MATCH, "Increasing term_info[" << query->tname << "] "
+		      "to (" << i->second.termfreq << ", " <<
+		      i->second.termweight << ")");
+	}
     }
 
     LeafPostList * pl = db->open_post_list(query->tname);
