@@ -221,6 +221,9 @@ MultiMatch::MultiMatch(const Xapian::Database &db_,
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 	    RemoteDatabase *rem_db = subdb->as_remotedatabase();
 	    if (rem_db) {
+		if (sorter) {
+		    throw Xapian::UnimplementedError("Xapian::Sorter not supported for the remote backend");
+		}
 		rem_db->set_query(query, qlen, collapse_key, order, sort_key,
 				  sort_by, sort_value_forward, percent_cutoff,
 				  weight_cutoff, weight, subrsets[i]);
