@@ -594,15 +594,18 @@ QueryParser::Internal::parse_query(const string &qs, unsigned flags,
     unsigned newprev = ' ';
 main_lex_loop:
     enum {
-	DEFAULT, IN_QUOTES, IN_PREFIXED_QUOTES, IN_PHRASED_TERM, IN_GROUP
+	DEFAULT, IN_QUOTES, IN_PREFIXED_QUOTES, IN_PHRASED_TERM, IN_GROUP,
+	EXPLICIT_SYNONYM
     } mode = DEFAULT;
     while (it != end && !state.error) {
 	bool last_was_operator = false;
+	if (mode == EXPLICIT_SYNONYM) mode = DEFAULT;
 	if (false) {
 just_had_operator:
 	    if (it == end) break;
-	    last_was_operator = true;
 	    mode = DEFAULT;
+just_had_synonym_operator:
+	    last_was_operator = true;
 	}
 	if (mode == IN_PHRASED_TERM) mode = DEFAULT;
 	if (is_whitespace(*it)) {
@@ -741,17 +744,18 @@ just_had_operator:
 	      case '~': // Synonym expansion.
 		// Ignore at the end of the query string.
 		if (it == end) goto done;
-		if (prev > ' ' && prev != '+' && prev != '-' && prev != '(') {
-		    // Or if not after whitespace, +, -, or an open bracket.
-		    break;
-		}
-		if (!is_wordchar(*it)) {
-		    // Ignore if not followed by a word character.
-		    break;
-		}
 		if (mode == DEFAULT && (flags & FLAG_SYNONYM)) {
+		    if (prev > ' ' && strchr("+-(", prev) == NULL) {
+			// Or if not after whitespace, +, -, or an open bracket.
+			break;
+		    }
+		    if (!is_wordchar(*it)) {
+			// Ignore if not followed by a word character.
+			break;
+		    }
 		    Parse(pParser, SYNONYM, NULL, &state);
-		    goto just_had_operator;
+		    mode = EXPLICIT_SYNONYM;
+		    goto just_had_synonym_operator;
 		}
 		break;
 	    }
@@ -767,7 +771,8 @@ just_had_operator:
 
 	// A term, a prefix, or a boolean operator.
 	const PrefixInfo * prefixinfo = NULL;
-	if ((mode == DEFAULT || mode == IN_GROUP) && !prefixmap.empty()) {
+	if ((mode == DEFAULT || mode == IN_GROUP || mode == EXPLICIT_SYNONYM) &&
+	    !prefixmap.empty()) {
 	    // Check for a fieldname prefix (e.g. title:historical).
 	    Utf8Iterator p = find_if(it, end, is_not_wordchar);
 	    if (p != end && *p == ':' && ++p != end && *p > ' ' && *p != ')') {
@@ -785,7 +790,8 @@ just_had_operator:
 
 		    if (prefixinfo->filter) {
 			// Drop out of IN_GROUP if we're in it.
-			mode = DEFAULT;
+			if (mode == IN_GROUP)
+			    mode = DEFAULT;
 			// Can't boolean filter prefix a subexpression or
 			// phrase; just use anything following the prefix
 			// until the next space or ')' as part of the boolean
@@ -1201,6 +1207,7 @@ TermGroup::as_group(State *state) const
 	}
     }
     delete this;
+    if (subqs.empty()) return NULL;
     return new Query(default_op, subqs.begin(), subqs.end());
 }
 
@@ -1330,7 +1337,7 @@ class TermList {
 	delete B;\
     } while (0)
 
-#line 1335 "queryparser/queryparser_internal.cc"
+#line 1342 "queryparser/queryparser_internal.cc"
 /* Next is all token values, in a form suitable for use by makeheaders.
 ** This section will be null unless lemon is run with the -m switch.
 */
@@ -1758,9 +1765,9 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 19:
     case 20:
     case 21:
-#line 1330 "queryparser/queryparser.lemony"
+#line 1337 "queryparser/queryparser.lemony"
 {delete (yypminor->yy0);}
-#line 1765 "queryparser/queryparser_internal.cc"
+#line 1772 "queryparser/queryparser_internal.cc"
       break;
     case 24:
     case 25:
@@ -1768,28 +1775,28 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 28:
     case 30:
     case 31:
-#line 1401 "queryparser/queryparser.lemony"
+#line 1408 "queryparser/queryparser.lemony"
 {delete (yypminor->yy13);}
-#line 1775 "queryparser/queryparser_internal.cc"
+#line 1782 "queryparser/queryparser_internal.cc"
       break;
     case 27:
     case 29:
-#line 1494 "queryparser/queryparser.lemony"
+#line 1501 "queryparser/queryparser.lemony"
 {delete (yypminor->yy9);}
-#line 1781 "queryparser/queryparser_internal.cc"
+#line 1788 "queryparser/queryparser_internal.cc"
       break;
     case 32:
     case 33:
     case 35:
     case 36:
-#line 1695 "queryparser/queryparser.lemony"
+#line 1702 "queryparser/queryparser.lemony"
 {(yypminor->yy1)->destroy();}
-#line 1789 "queryparser/queryparser_internal.cc"
+#line 1796 "queryparser/queryparser_internal.cc"
       break;
     case 34:
-#line 1729 "queryparser/queryparser.lemony"
+#line 1736 "queryparser/queryparser.lemony"
 {(yypminor->yy60)->destroy();}
-#line 1794 "queryparser/queryparser_internal.cc"
+#line 1801 "queryparser/queryparser_internal.cc"
       break;
     default:  break;   /* If no destructor action specified: do nothing */
   }
@@ -2021,7 +2028,7 @@ static void yy_reduce(
   **     break;
   */
       case 0:
-#line 1383 "queryparser/queryparser.lemony"
+#line 1390 "queryparser/queryparser.lemony"
 {
     // Save the parsed query in the State structure so we can return it.
     if (yymsp[0].minor.yy13) {
@@ -2031,30 +2038,30 @@ static void yy_reduce(
 	state->query = Query();
     }
 }
-#line 2036 "queryparser/queryparser_internal.cc"
+#line 2043 "queryparser/queryparser_internal.cc"
         break;
       case 1:
-#line 1393 "queryparser/queryparser.lemony"
+#line 1400 "queryparser/queryparser.lemony"
 {
     // Handle a query string with no terms in.
     state->query = Query();
 }
-#line 2044 "queryparser/queryparser_internal.cc"
+#line 2051 "queryparser/queryparser_internal.cc"
         break;
       case 2:
       case 8:
-#line 1404 "queryparser/queryparser.lemony"
+#line 1411 "queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy13; }
-#line 2050 "queryparser/queryparser_internal.cc"
+#line 2057 "queryparser/queryparser_internal.cc"
         break;
       case 3:
-#line 1407 "queryparser/queryparser.lemony"
+#line 1414 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy13, yymsp[-2].minor.yy13, Query::OP_AND, yymsp[0].minor.yy13, "AND");   yy_destructor(4,&yymsp[-1].minor);
 }
-#line 2056 "queryparser/queryparser_internal.cc"
+#line 2063 "queryparser/queryparser_internal.cc"
         break;
       case 4:
-#line 1409 "queryparser/queryparser.lemony"
+#line 1416 "queryparser/queryparser.lemony"
 {
     // 'NOT foo' -> '<alldocuments> NOT foo'
     if (!yymsp[-2].minor.yy13 && (state->flags & QueryParser::FLAG_PURE_NOT)) {
@@ -2063,39 +2070,39 @@ static void yy_reduce(
     BOOL_OP_TO_QUERY(yygotominor.yy13, yymsp[-2].minor.yy13, Query::OP_AND_NOT, yymsp[0].minor.yy13, "NOT");
   yy_destructor(5,&yymsp[-1].minor);
 }
-#line 2068 "queryparser/queryparser_internal.cc"
+#line 2075 "queryparser/queryparser_internal.cc"
         break;
       case 5:
-#line 1418 "queryparser/queryparser.lemony"
+#line 1425 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy13, yymsp[-3].minor.yy13, Query::OP_AND_NOT, yymsp[0].minor.yy13, "AND NOT");   yy_destructor(4,&yymsp[-2].minor);
   yy_destructor(5,&yymsp[-1].minor);
 }
-#line 2075 "queryparser/queryparser_internal.cc"
+#line 2082 "queryparser/queryparser_internal.cc"
         break;
       case 6:
-#line 1421 "queryparser/queryparser.lemony"
+#line 1428 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy13, yymsp[-2].minor.yy13, Query::OP_OR, yymsp[0].minor.yy13, "OR");   yy_destructor(2,&yymsp[-1].minor);
 }
-#line 2081 "queryparser/queryparser_internal.cc"
+#line 2088 "queryparser/queryparser_internal.cc"
         break;
       case 7:
-#line 1424 "queryparser/queryparser.lemony"
+#line 1431 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy13, yymsp[-2].minor.yy13, Query::OP_XOR, yymsp[0].minor.yy13, "XOR");   yy_destructor(3,&yymsp[-1].minor);
 }
-#line 2087 "queryparser/queryparser_internal.cc"
+#line 2094 "queryparser/queryparser_internal.cc"
         break;
       case 9:
-#line 1433 "queryparser/queryparser.lemony"
+#line 1440 "queryparser/queryparser.lemony"
 {
     // Set the argument to NULL, which enables the bool_arg-using rules in
     // expr above to report uses of AND, OR, etc which don't have two
     // arguments.
     yygotominor.yy13 = NULL;
 }
-#line 2097 "queryparser/queryparser_internal.cc"
+#line 2104 "queryparser/queryparser_internal.cc"
         break;
       case 10:
-#line 1445 "queryparser/queryparser.lemony"
+#line 1452 "queryparser/queryparser.lemony"
 {
     yygotominor.yy13 = yymsp[0].minor.yy9->query;
     yymsp[0].minor.yy9->query = NULL;
@@ -2134,19 +2141,19 @@ static void yy_reduce(
     // FIXME what if yygotominor.yy13 && yygotominor.yy13->empty() (all terms are stopwords)?
     delete yymsp[0].minor.yy9;
 }
-#line 2139 "queryparser/queryparser_internal.cc"
+#line 2146 "queryparser/queryparser_internal.cc"
         break;
       case 11:
       case 29:
       case 31:
-#line 1484 "queryparser/queryparser.lemony"
+#line 1491 "queryparser/queryparser.lemony"
 {
     yygotominor.yy13 = yymsp[0].minor.yy13;
 }
-#line 2148 "queryparser/queryparser_internal.cc"
+#line 2155 "queryparser/queryparser_internal.cc"
         break;
       case 12:
-#line 1496 "queryparser/queryparser.lemony"
+#line 1503 "queryparser/queryparser.lemony"
 {
     Query range;
     Xapian::valueno valno = state->value_range(range, yymsp[-1].minor.yy0, yymsp[0].minor.yy0);
@@ -2157,10 +2164,10 @@ static void yy_reduce(
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->filter[filter_group_id(valno)] = range;
 }
-#line 2162 "queryparser/queryparser_internal.cc"
+#line 2169 "queryparser/queryparser_internal.cc"
         break;
       case 13:
-#line 1507 "queryparser/queryparser.lemony"
+#line 1514 "queryparser/queryparser.lemony"
 {
     Query range;
     Xapian::valueno valno = state->value_range(range, yymsp[-1].minor.yy0, yymsp[0].minor.yy0);
@@ -2172,28 +2179,28 @@ static void yy_reduce(
     Query & q = yygotominor.yy9->filter[filter_group_id(valno)];
     q = Query(Query::OP_OR, q, range);
 }
-#line 2177 "queryparser/queryparser_internal.cc"
+#line 2184 "queryparser/queryparser_internal.cc"
         break;
       case 14:
-#line 1519 "queryparser/queryparser.lemony"
+#line 1526 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->query = yymsp[-1].minor.yy13;
     if (yymsp[0].minor.yy13) add_to_query(yygotominor.yy9->query, state->default_op(), yymsp[0].minor.yy13);
 }
-#line 2186 "queryparser/queryparser_internal.cc"
+#line 2193 "queryparser/queryparser_internal.cc"
         break;
       case 15:
-#line 1525 "queryparser/queryparser.lemony"
+#line 1532 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = yymsp[-1].minor.yy9;
     // If yymsp[0].minor.yy13 is a stopword, there's nothing to do here.
     if (yymsp[0].minor.yy13) add_to_query(yygotominor.yy9->query, state->default_op(), yymsp[0].minor.yy13);
 }
-#line 2195 "queryparser/queryparser_internal.cc"
+#line 2202 "queryparser/queryparser_internal.cc"
         break;
       case 16:
-#line 1531 "queryparser/queryparser.lemony"
+#line 1538 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     if (state->default_op() == Query::OP_AND) {
@@ -2203,10 +2210,10 @@ static void yy_reduce(
     }
   yy_destructor(8,&yymsp[-1].minor);
 }
-#line 2208 "queryparser/queryparser_internal.cc"
+#line 2215 "queryparser/queryparser_internal.cc"
         break;
       case 17:
-#line 1540 "queryparser/queryparser.lemony"
+#line 1547 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = yymsp[-2].minor.yy9;
     if (state->default_op() == Query::OP_AND) {
@@ -2219,57 +2226,57 @@ static void yy_reduce(
     }
   yy_destructor(8,&yymsp[-1].minor);
 }
-#line 2224 "queryparser/queryparser_internal.cc"
+#line 2231 "queryparser/queryparser_internal.cc"
         break;
       case 18:
-#line 1552 "queryparser/queryparser.lemony"
+#line 1559 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->hate = yymsp[0].minor.yy13;
   yy_destructor(9,&yymsp[-1].minor);
 }
-#line 2233 "queryparser/queryparser_internal.cc"
+#line 2240 "queryparser/queryparser_internal.cc"
         break;
       case 19:
-#line 1557 "queryparser/queryparser.lemony"
+#line 1564 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = yymsp[-2].minor.yy9;
     add_to_query(yygotominor.yy9->hate, Query::OP_OR, yymsp[0].minor.yy13);
   yy_destructor(9,&yymsp[-1].minor);
 }
-#line 2242 "queryparser/queryparser_internal.cc"
+#line 2249 "queryparser/queryparser_internal.cc"
         break;
       case 20:
-#line 1562 "queryparser/queryparser.lemony"
+#line 1569 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->hate = new Query(yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
   yy_destructor(9,&yymsp[-1].minor);
 }
-#line 2252 "queryparser/queryparser_internal.cc"
+#line 2259 "queryparser/queryparser_internal.cc"
         break;
       case 21:
-#line 1568 "queryparser/queryparser.lemony"
+#line 1575 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = yymsp[-2].minor.yy9;
     add_to_query(yygotominor.yy9->hate, Query::OP_OR, yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
   yy_destructor(9,&yymsp[-1].minor);
 }
-#line 2262 "queryparser/queryparser_internal.cc"
+#line 2269 "queryparser/queryparser_internal.cc"
         break;
       case 22:
-#line 1574 "queryparser/queryparser.lemony"
+#line 1581 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->filter[yymsp[0].minor.yy0->get_filter_group_id()] = yymsp[0].minor.yy0->get_query();
     delete yymsp[0].minor.yy0;
 }
-#line 2271 "queryparser/queryparser_internal.cc"
+#line 2278 "queryparser/queryparser_internal.cc"
         break;
       case 23:
-#line 1580 "queryparser/queryparser.lemony"
+#line 1587 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = yymsp[-1].minor.yy9;
     // We OR filters with the same prefix...
@@ -2277,10 +2284,10 @@ static void yy_reduce(
     q = Query(Query::OP_OR, q, yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
 }
-#line 2282 "queryparser/queryparser_internal.cc"
+#line 2289 "queryparser/queryparser_internal.cc"
         break;
       case 24:
-#line 1588 "queryparser/queryparser.lemony"
+#line 1595 "queryparser/queryparser.lemony"
 {
     // LOVE BOOLEAN_FILTER(yymsp[0].minor.yy0) is just the same as BOOLEAN_FILTER
     yygotominor.yy9 = new ProbQuery;
@@ -2288,10 +2295,10 @@ static void yy_reduce(
     delete yymsp[0].minor.yy0;
   yy_destructor(8,&yymsp[-1].minor);
 }
-#line 2293 "queryparser/queryparser_internal.cc"
+#line 2300 "queryparser/queryparser_internal.cc"
         break;
       case 25:
-#line 1595 "queryparser/queryparser.lemony"
+#line 1602 "queryparser/queryparser.lemony"
 {
     // LOVE BOOLEAN_FILTER(yymsp[0].minor.yy0) is just the same as BOOLEAN_FILTER
     yygotominor.yy9 = yymsp[-2].minor.yy9;
@@ -2301,23 +2308,23 @@ static void yy_reduce(
     delete yymsp[0].minor.yy0;
   yy_destructor(8,&yymsp[-1].minor);
 }
-#line 2306 "queryparser/queryparser_internal.cc"
+#line 2313 "queryparser/queryparser_internal.cc"
         break;
       case 26:
-#line 1610 "queryparser/queryparser.lemony"
+#line 1617 "queryparser/queryparser.lemony"
 { yygotominor.yy9 = yymsp[0].minor.yy9; }
-#line 2311 "queryparser/queryparser_internal.cc"
+#line 2318 "queryparser/queryparser_internal.cc"
         break;
       case 27:
-#line 1612 "queryparser/queryparser.lemony"
+#line 1619 "queryparser/queryparser.lemony"
 {
     yygotominor.yy9 = new ProbQuery;
     yygotominor.yy9->query = yymsp[0].minor.yy13;
 }
-#line 2319 "queryparser/queryparser_internal.cc"
+#line 2326 "queryparser/queryparser_internal.cc"
         break;
       case 28:
-#line 1626 "queryparser/queryparser.lemony"
+#line 1633 "queryparser/queryparser.lemony"
 {
     if (state->is_stopword(yymsp[0].minor.yy0)) {
 	yygotominor.yy13 = NULL;
@@ -2327,117 +2334,117 @@ static void yy_reduce(
     }
     delete yymsp[0].minor.yy0;
 }
-#line 2332 "queryparser/queryparser_internal.cc"
+#line 2339 "queryparser/queryparser_internal.cc"
         break;
       case 30:
-#line 1645 "queryparser/queryparser.lemony"
+#line 1652 "queryparser/queryparser.lemony"
 {
     yygotominor.yy13 = new Query(yymsp[0].minor.yy0->get_query_with_auto_synonyms());
     delete yymsp[0].minor.yy0;
 }
-#line 2340 "queryparser/queryparser_internal.cc"
+#line 2347 "queryparser/queryparser_internal.cc"
         break;
       case 32:
-#line 1662 "queryparser/queryparser.lemony"
+#line 1669 "queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy0->as_wildcarded_query(state); }
-#line 2345 "queryparser/queryparser_internal.cc"
+#line 2352 "queryparser/queryparser_internal.cc"
         break;
       case 33:
-#line 1665 "queryparser/queryparser.lemony"
+#line 1672 "queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy0->as_partial_query(state); }
-#line 2350 "queryparser/queryparser_internal.cc"
+#line 2357 "queryparser/queryparser_internal.cc"
         break;
       case 34:
-#line 1668 "queryparser/queryparser.lemony"
+#line 1675 "queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[-1].minor.yy1->as_phrase_query();   yy_destructor(19,&yymsp[-2].minor);
   yy_destructor(19,&yymsp[0].minor);
 }
-#line 2357 "queryparser/queryparser_internal.cc"
+#line 2364 "queryparser/queryparser_internal.cc"
         break;
       case 35:
-#line 1671 "queryparser/queryparser.lemony"
+#line 1678 "queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy1->as_phrase_query(); }
-#line 2362 "queryparser/queryparser_internal.cc"
+#line 2369 "queryparser/queryparser_internal.cc"
         break;
       case 36:
-#line 1673 "queryparser/queryparser.lemony"
+#line 1680 "queryparser/queryparser.lemony"
 {
     yygotominor.yy13 = yymsp[0].minor.yy60->as_group(state);
 }
-#line 2369 "queryparser/queryparser_internal.cc"
+#line 2376 "queryparser/queryparser_internal.cc"
         break;
       case 37:
-#line 1678 "queryparser/queryparser.lemony"
+#line 1685 "queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy1->as_near_query(); }
-#line 2374 "queryparser/queryparser_internal.cc"
+#line 2381 "queryparser/queryparser_internal.cc"
         break;
       case 38:
-#line 1681 "queryparser/queryparser.lemony"
+#line 1688 "queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[0].minor.yy1->as_adj_query(); }
-#line 2379 "queryparser/queryparser_internal.cc"
+#line 2386 "queryparser/queryparser_internal.cc"
         break;
       case 39:
-#line 1684 "queryparser/queryparser.lemony"
+#line 1691 "queryparser/queryparser.lemony"
 { yygotominor.yy13 = yymsp[-1].minor.yy13;   yy_destructor(20,&yymsp[-2].minor);
   yy_destructor(21,&yymsp[0].minor);
 }
-#line 2386 "queryparser/queryparser_internal.cc"
+#line 2393 "queryparser/queryparser_internal.cc"
         break;
       case 40:
-#line 1686 "queryparser/queryparser.lemony"
+#line 1693 "queryparser/queryparser.lemony"
 {
     yygotominor.yy13 = new Query(yymsp[0].minor.yy0->get_query_with_synonyms());
     delete yymsp[0].minor.yy0;
   yy_destructor(10,&yymsp[-1].minor);
 }
-#line 2395 "queryparser/queryparser_internal.cc"
+#line 2402 "queryparser/queryparser_internal.cc"
         break;
       case 41:
-#line 1697 "queryparser/queryparser.lemony"
+#line 1704 "queryparser/queryparser.lemony"
 {
     yygotominor.yy1 = new TermList;
     yygotominor.yy1->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2403 "queryparser/queryparser_internal.cc"
+#line 2410 "queryparser/queryparser_internal.cc"
         break;
       case 42:
       case 44:
-#line 1702 "queryparser/queryparser.lemony"
+#line 1709 "queryparser/queryparser.lemony"
 {
     yygotominor.yy1 = yymsp[-1].minor.yy1;
     yygotominor.yy1->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2412 "queryparser/queryparser_internal.cc"
+#line 2419 "queryparser/queryparser_internal.cc"
         break;
       case 43:
-#line 1714 "queryparser/queryparser.lemony"
+#line 1721 "queryparser/queryparser.lemony"
 {
     yygotominor.yy1 = new TermList;
     yygotominor.yy1->add_positional_term(yymsp[-1].minor.yy0);
     yygotominor.yy1->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2421 "queryparser/queryparser_internal.cc"
+#line 2428 "queryparser/queryparser_internal.cc"
         break;
       case 45:
-#line 1731 "queryparser/queryparser.lemony"
+#line 1738 "queryparser/queryparser.lemony"
 {
     yygotominor.yy60 = new TermGroup;
     yygotominor.yy60->add_term(yymsp[-1].minor.yy0);
     yygotominor.yy60->add_term(yymsp[0].minor.yy0);
 }
-#line 2430 "queryparser/queryparser_internal.cc"
+#line 2437 "queryparser/queryparser_internal.cc"
         break;
       case 46:
-#line 1737 "queryparser/queryparser.lemony"
+#line 1744 "queryparser/queryparser.lemony"
 {
     yygotominor.yy60 = yymsp[-1].minor.yy60;
     yygotominor.yy60->add_term(yymsp[0].minor.yy0);
 }
-#line 2438 "queryparser/queryparser_internal.cc"
+#line 2445 "queryparser/queryparser_internal.cc"
         break;
       case 47:
       case 49:
-#line 1748 "queryparser/queryparser.lemony"
+#line 1755 "queryparser/queryparser.lemony"
 {
     yygotominor.yy1 = new TermList;
     yygotominor.yy1->add_positional_term(yymsp[-2].minor.yy0);
@@ -2447,11 +2454,11 @@ static void yy_reduce(
 	delete yymsp[-1].minor.yy0;
     }
 }
-#line 2452 "queryparser/queryparser_internal.cc"
+#line 2459 "queryparser/queryparser_internal.cc"
         break;
       case 48:
       case 50:
-#line 1758 "queryparser/queryparser.lemony"
+#line 1765 "queryparser/queryparser.lemony"
 {
     yygotominor.yy1 = yymsp[-2].minor.yy1;
     yygotominor.yy1->add_positional_term(yymsp[0].minor.yy0);
@@ -2460,7 +2467,7 @@ static void yy_reduce(
 	delete yymsp[-1].minor.yy0;
     }
 }
-#line 2465 "queryparser/queryparser_internal.cc"
+#line 2472 "queryparser/queryparser_internal.cc"
         break;
   }
   yygoto = yyRuleInfo[yyruleno].lhs;
@@ -2485,11 +2492,11 @@ static void yy_parse_failed(
   while( !yypParser->yystack.empty() ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will be executed whenever the
   ** parser fails */
-#line 1334 "queryparser/queryparser.lemony"
+#line 1341 "queryparser/queryparser.lemony"
 
     // If we've not already set an error message, set a default one.
     if (!state->error) state->error = "parse error";
-#line 2495 "queryparser/queryparser_internal.cc"
+#line 2502 "queryparser/queryparser_internal.cc"
   ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
