@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
  * Copyright 2006,2007 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -27,7 +27,6 @@
 
 #include <algorithm>
 #include <fstream>
-#include <iomanip>
 #include <map>
 #include <string>
 #include <vector>
@@ -398,7 +397,7 @@ DEFINE_TESTCASE(matchfunctor3, backend) {
     Xapian::Enquire enquire(db);
     enquire.set_query(Xapian::Query(""));
     enquire.set_collapse_key(12);
-    enquire.set_sort_by_value(11);
+    enquire.set_sort_by_value(11, true);
 
     myMatchDecider2 myfunctor;
 
@@ -602,7 +601,7 @@ DEFINE_TESTCASE(collapsekey1, backend) {
 	Xapian::MSetIterator i = mymset.begin();
 	for ( ; i != mymset.end(); ++i) {
 	    string value = i.get_document().get_value(value_no);
-	    TEST(values[value] == 0 || value == "");
+	    TEST(values[value] == 0 || value.empty());
 	    values[value] = *i;
 	}
     }
@@ -657,7 +656,7 @@ DEFINE_TESTCASE(collapsekey3, backend) {
 	Xapian::MSetIterator i = mymset.begin();
 	for ( ; i != mymset.end(); ++i) {
 	    string value = i.get_document().get_value(value_no);
-	    TEST(values[value] == 0 || value == "");
+	    TEST(values[value] == 0 || value.empty());
 	    values[value] = *i;
 	}
     }
@@ -679,7 +678,7 @@ DEFINE_TESTCASE(collapsekey3, backend) {
 	Xapian::MSetIterator i = mymset.begin();
 	for ( ; i != mymset.end(); ++i) {
 	    string value = i.get_document().get_value(value_no);
-	    TEST(values[value] == 0 || value == "");
+	    TEST(values[value] == 0 || value.empty());
 	    values[value] = *i;
 	}
     }
@@ -708,7 +707,7 @@ DEFINE_TESTCASE(collapsekey4, backend) {
 	Xapian::MSetIterator i = mymset.begin();
 	for ( ; i != mymset.end(); ++i) {
 	    string value = i.get_document().get_value(value_no);
-	    TEST(values[value] == 0 || value == "");
+	    TEST(values[value] == 0 || value.empty());
 	    values[value] = *i;
 	}
     }
@@ -743,8 +742,8 @@ DEFINE_TESTCASE(allterms1, backend) {
     Xapian::Database db(get_database("apitest_allterms"));
     Xapian::TermIterator ati = db.allterms_begin();
     TEST(ati != db.allterms_end());
-    TEST(*ati == "one");
-    TEST(ati.get_termfreq() == 1);
+    TEST_EQUAL(*ati, "one");
+    TEST_EQUAL(ati.get_termfreq(), 1);
 
     Xapian::TermIterator ati2 = ati;
 
@@ -1094,14 +1093,20 @@ DEFINE_TESTCASE(postlist2, backend) {
     p = db.postlist_begin("this");
     Xapian::PostingIterator pend = db.postlist_end("this");
 
+    TEST(p.get_description() != "Xapian::PostingIterator(pos=END)");
+
     // test operator= creates a copy which compares equal
     Xapian::PostingIterator p_copy = p;
     TEST_EQUAL(p, p_copy);
 
+    TEST(p_copy.get_description() != "Xapian::PostingIterator(pos=END)");
+
     // test copy constructor creates a copy which compares equal
     Xapian::PostingIterator p_clone(p);
     TEST_EQUAL(p, p_clone);
 
+    TEST(p_clone.get_description() != "Xapian::PostingIterator(pos=END)");
+
     vector<Xapian::docid> v(p, pend);
 
     p = db.postlist_begin("this");
@@ -1113,6 +1118,12 @@ DEFINE_TESTCASE(postlist2, backend) {
 	p++;
     }
     TEST_EQUAL(p, pend);
+
+    TEST_STRINGS_EQUAL(p.get_description(),
+		       "Xapian::PostingIterator(pos=END)");
+    TEST_STRINGS_EQUAL(pend.get_description(),
+		       "Xapian::PostingIterator(pos=END)");
+
     return true;
 }
 
@@ -1170,7 +1181,8 @@ DEFINE_TESTCASE(postlist6, backend) {
     TEST(i != db.postlist_end("this"));
     while (i != db.postlist_end("this")) {
 	TEST_EQUAL(i.get_doclength(), db.get_doclength(*i));
-	i++;
+	TEST_REL(i.get_wdf(),<=,i.get_doclength());
+	++i;
     }
     return true;
 }
@@ -1208,7 +1220,7 @@ DEFINE_TESTCASE(sortvalue1, backend) {
     for (int pass = 1; pass <= 2; ++pass) {
 	for (Xapian::valueno value_no = 1; value_no < 7; ++value_no) {
 	    tout << "Sorting on value " << value_no << endl;
-	    enquire.set_sort_by_value(value_no);
+	    enquire.set_sort_by_value(value_no, true);
 	    Xapian::MSet allbset = enquire.get_mset(0, 100);
 	    Xapian::MSet partbset1 = enquire.get_mset(0, 3);
 	    Xapian::MSet partbset2 = enquire.get_mset(3, 97);
@@ -1375,6 +1387,7 @@ DEFINE_TESTCASE(quartzdatabaseopen1, quartz) {
 
 // tests that specifying a nonexistent input file throws an exception.
 DEFINE_TESTCASE(flintdatabaseopeningerror1, flint) {
+#ifdef XAPIAN_HAS_FLINT_BACKEND
     mkdir(".flint", 0755);
 
     TEST_EXCEPTION(Xapian::DatabaseOpeningError,
@@ -1397,12 +1410,14 @@ DEFINE_TESTCASE(flintdatabaseopeningerror1, flint) {
 		   Xapian::Flint::open(".flint/somefile", Xapian::DB_CREATE_OR_OPEN));
     TEST_EXCEPTION(Xapian::DatabaseCreateError,
 		   Xapian::Flint::open(".flint/somefile", Xapian::DB_CREATE_OR_OVERWRITE));
+#endif
 
     return true;
 }
 
 /// Tests that appropriate error is thrown for database format change.
 DEFINE_TESTCASE(flintdatabaseformaterror1, flint) {
+#ifdef XAPIAN_HAS_FLINT_BACKEND
     string dbdir = test_driver::get_srcdir();
     dbdir += "/testdata/flint-0.9.9";
 
@@ -1418,6 +1433,7 @@ DEFINE_TESTCASE(flintdatabaseformaterror1, flint) {
     // Clean up the "flintlock" file that we will have created while trying
     // to open the database.
     unlink((dbdir + "/flintlock").c_str());
+#endif
 
     return true;
 }
@@ -1425,6 +1441,7 @@ DEFINE_TESTCASE(flintdatabaseformaterror1, flint) {
 /// Test that an old database can be successfully overwritten when using
 // Xapian::DB_CREATE_OR_OVERWRITE.
 DEFINE_TESTCASE(flintdatabaseformaterror2, flint) {
+#ifdef XAPIAN_HAS_FLINT_BACKEND
     string flint099 = test_driver::get_srcdir();
     flint099 += "/testdata/flint-0.9.9";
 
@@ -1441,12 +1458,14 @@ DEFINE_TESTCASE(flintdatabaseformaterror2, flint) {
 
     // Also test when explicitly opening as a flint database.
     (void)Xapian::Flint::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
+#endif
 
     return true;
 }
 
 // regression test for not releasing lock on error.
 DEFINE_TESTCASE(flintdatabaseformaterror3, flint) {
+#ifdef XAPIAN_HAS_FLINT_BACKEND
     string flint099 = test_driver::get_srcdir();
     flint099 += "/testdata/flint-0.9.9";
 
@@ -1462,12 +1481,14 @@ DEFINE_TESTCASE(flintdatabaseformaterror3, flint) {
     // This used to throw a DatabaseLockError: "Unable to acquire database
     // write lock on .flint/formatdb: already locked"
     Xapian::WritableDatabase(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
+#endif
 
     return true;
 }
 
 // Test that 1.0.2 and later can open 1.0.1 databases.
 DEFINE_TESTCASE(flintbackwardcompat1, flint) {
+#ifdef XAPIAN_HAS_FLINT_BACKEND
     string flint101 = test_driver::get_srcdir();
     flint101 += "/testdata/flint-1.0.1";
 
@@ -1488,12 +1509,14 @@ DEFINE_TESTCASE(flintbackwardcompat1, flint) {
 	Xapian::WritableDatabase db(dbdir, Xapian::DB_OPEN);
 	TEST_EQUAL(db.get_doccount(), 0);
     }
+#endif
 
     return true;
 }
 
 // Test that 1.0.3 and later can open 1.0.2 databases.
 DEFINE_TESTCASE(flintbackwardcompat2, flint) {
+#ifdef XAPIAN_HAS_FLINT_BACKEND
     string flint102 = test_driver::get_srcdir();
     flint102 += "/testdata/flint-1.0.2";
 
@@ -1514,12 +1537,14 @@ DEFINE_TESTCASE(flintbackwardcompat2, flint) {
 	Xapian::WritableDatabase db(dbdir, Xapian::DB_OPEN);
 	TEST_EQUAL(db.get_doccount(), 0);
     }
+#endif
 
     return true;
 }
 
 /// Test opening of a flint database
 DEFINE_TESTCASE(flintdatabaseopen1, flint) {
+#ifdef XAPIAN_HAS_FLINT_BACKEND
     const string dbdir = ".flint/test_flintdatabaseopen1";
     mkdir(".flint", 0755);
 
@@ -1569,6 +1594,7 @@ DEFINE_TESTCASE(flintdatabaseopen1, flint) {
 	    Xapian::Flint::open(dbdir, Xapian::DB_OPEN);
 	Xapian::Flint::open(dbdir);
     }
+#endif
 
     return true;
 }
@@ -1579,7 +1605,7 @@ DEFINE_TESTCASE(flintdatabaseopen1, flint) {
 // set_sort_by_relevance_then_value
 DEFINE_TESTCASE(sortrel1, backend) {
     Xapian::Enquire enquire(get_database("apitest_sortrel"));
-    enquire.set_sort_by_value(1);
+    enquire.set_sort_by_value(1, true);
     enquire.set_query(Xapian::Query("woman"));
 
     const Xapian::docid order1[] = { 1,2,3,4,5,6,7,8,9 };
@@ -1601,7 +1627,7 @@ DEFINE_TESTCASE(sortrel1, backend) {
 	TEST_EQUAL(*mset[i], order1[i]);
     }
 
-    enquire.set_sort_by_value_then_relevance(1);
+    enquire.set_sort_by_value_then_relevance(1, true);
 
     mset = enquire.get_mset(0, 10);
     TEST_EQUAL(mset.size(), sizeof(order2) / sizeof(Xapian::docid));
@@ -1609,7 +1635,7 @@ DEFINE_TESTCASE(sortrel1, backend) {
 	TEST_EQUAL(*mset[i], order2[i]);
     }
 
-    enquire.set_sort_by_value(1);
+    enquire.set_sort_by_value(1, true);
 
     mset = enquire.get_mset(0, 10);
     TEST_EQUAL(mset.size(), sizeof(order1) / sizeof(Xapian::docid));
@@ -1617,7 +1643,7 @@ DEFINE_TESTCASE(sortrel1, backend) {
 	TEST_EQUAL(*mset[i], order1[i]);
     }
 
-    enquire.set_sort_by_value_then_relevance(1);
+    enquire.set_sort_by_value_then_relevance(1, true);
     enquire.set_docid_order(Xapian::Enquire::DESCENDING);
 
     mset = enquire.get_mset(0, 10);
@@ -1626,7 +1652,7 @@ DEFINE_TESTCASE(sortrel1, backend) {
 	TEST_EQUAL(*mset[i], order2[i]);
     }
 
-    enquire.set_sort_by_value(1);
+    enquire.set_sort_by_value(1, true);
     enquire.set_docid_order(Xapian::Enquire::DESCENDING);
 
     mset = enquire.get_mset(0, 10);
@@ -1667,7 +1693,7 @@ DEFINE_TESTCASE(sortrel1, backend) {
 	TEST_EQUAL(*mset[i], order7[i]);
     }
 
-    enquire.set_sort_by_relevance_then_value(1);
+    enquire.set_sort_by_relevance_then_value(1, true);
     enquire.set_docid_order(Xapian::Enquire::ASCENDING);
     mset = enquire.get_mset(0, 10);
     TEST_EQUAL(mset.size(), sizeof(order8) / sizeof(Xapian::docid));
@@ -1675,7 +1701,7 @@ DEFINE_TESTCASE(sortrel1, backend) {
 	TEST_EQUAL(*mset[i], order8[i]);
     }
 
-    enquire.set_sort_by_relevance_then_value(1);
+    enquire.set_sort_by_relevance_then_value(1, true);
     enquire.set_docid_order(Xapian::Enquire::DESCENDING);
     mset = enquire.get_mset(0, 10);
     TEST_EQUAL(mset.size(), sizeof(order8) / sizeof(Xapian::docid));
@@ -1811,7 +1837,8 @@ DEFINE_TESTCASE(userweight1, backend && !remote) {
 
 // tests MatchAll queries
 // This is a regression test, which failed with assertion failures in
-// revision 9094.
+// revision 9094.  Also check that the results aren't ranked by relevance
+// (regression test for bug fixed in 1.0.9).
 DEFINE_TESTCASE(matchall1, backend) {
     Xapian::Database db(get_database("apitest_simpledata"));
     Xapian::Enquire enquire(db);
@@ -1826,5 +1853,11 @@ DEFINE_TESTCASE(matchall1, backend) {
     mset = enquire.get_mset(0, 10);
     TEST_EQUAL(mset.get_matches_lower_bound(), db.get_doccount());
 
+    // Check that the results aren't ranked by relevance (fixed in 1.0.9).
+    TEST(mset.size() > 1);
+    TEST_EQUAL(mset[mset.size() - 1].get_weight(), 0);
+    TEST_EQUAL(*mset[0], 1);
+    TEST_EQUAL(*mset[mset.size() - 1], mset.size());
+ 
     return true;
 }
