@@ -270,6 +270,12 @@ DEFINE_TESTCASE(stemlangs1, !backend) {
 	string language = langs.substr(i, spc - i);
 	tout << "checking language code '" << language << "' works" << endl;
 	Xapian::Stem stemmer(language);
+	if (language.size() > 2) {
+	    string expected("Xapian::Stem(");
+	    expected += language;
+	    expected += ')';
+	    TEST_EQUAL(stemmer.get_description(), expected);
+	}
 
 	if (spc == string::npos) break;
 	i = spc + 1;
@@ -379,7 +385,8 @@ DEFINE_TESTCASE(scaleweight4, !backend) {
     // Factor is a double which, when multiplied by its reciprocal, doesn't
     // give exactly 1.0
     double factor = 179.76931348623157e306;
-    double nearly1 = factor * (1.0 / factor);
+    volatile double recip = 1.0 / factor;
+    double nearly1 = factor * recip;
 
     TEST_NOT_EQUAL(nearly1, 1.0);
     Xapian::Query foo("foo");
@@ -389,9 +396,27 @@ DEFINE_TESTCASE(scaleweight4, !backend) {
     return true;
 }
 
+// Regression test - RSet::get_description() gave a malformed answer in 1.0.7.
+DEFINE_TESTCASE(rset4, !backend) {
+    Xapian::RSet rset;
+    rset.add_document(1);
+    // In 1.0.7 this gave: RSet(RSet(RSet::Internal(, 1))
+    TEST_STRINGS_EQUAL(rset.get_description(), "RSet(RSet::Internal(1))");
+    return true;
+}
+
 // Check that Query(OP_VALUE_GE, 0, "") -> Query::MatchAll.
 DEFINE_TESTCASE(opvaluege1, !backend) {
     Xapian::Query query(Xapian::Query::OP_VALUE_GE, 0, "");
     TEST_STRINGS_EQUAL(query.get_description(), Xapian::Query::MatchAll.get_description());
     return true;
 }
+
+// Regression test: asking for a metadata on empty Database used to segfault.
+DEFINE_TESTCASE(emptydb_metadata1, !backend) {
+    Xapian::Database db;
+    TEST_EQUAL(db.get_metadata("foo"), std::string());
+    TEST_EQUAL(db.metadata_keys_begin(),
+	       db.metadata_keys_end());
+    return true;
+}
