@@ -27,19 +27,20 @@
 
 #include "api_wrdb.h"
 
-#include <algorithm>
-#include <map>
-#include <string>
-
 #include <xapian.h>
+
+#include "backendmanager.h" // For XAPIAN_BIN_PATH.
 #include "testsuite.h"
 #include "testutils.h"
 #include "utils.h"
+#include "unixcmds.h"
 
 #include "apitest.h"
 
 #include <cmath>
-#include <list>
+#include <cstdlib>
+#include <map>
+#include <string>
 
 using namespace std;
 
@@ -804,6 +805,8 @@ DEFINE_TESTCASE(deldoc5, writable) {
 
     db.delete_document(2);
 
+    TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_document(2));
+
     db.flush();
 
     TEST_EXCEPTION(Xapian::DocNotFoundError, db.get_document(2));
@@ -1112,6 +1115,46 @@ DEFINE_TESTCASE(replacedoc5, writable) {
     return true;
 }
 
+// Test replacing a document while adding values, without changing anything
+// else.  Regression test for a bug introduced while implementing lazy update,
+// and also covers a few other code paths.
+DEFINE_TESTCASE(replacedoc6, writable) {
+    Xapian::WritableDatabase db = get_writable_database();
+
+    Xapian::Document doc;
+    Xapian::docid did = db.add_document(doc);
+    TEST_EQUAL(did, 1);
+    db.flush();
+
+    // Add document
+    doc = db.get_document(1);
+    TEST_EQUAL(doc.get_value(1), "");
+    TEST_EQUAL(doc.get_value(2), "");
+    doc.add_value(1, "banana1");
+    db.replace_document(1, doc);
+
+    doc = db.get_document(1);
+    TEST_EQUAL(doc.get_value(1), "banana1");
+    TEST_EQUAL(doc.get_value(2), "");
+    db.flush();
+
+    doc = db.get_document(1);
+    TEST_EQUAL(doc.get_value(1), "banana1");
+    TEST_EQUAL(doc.get_value(2), "");
+    doc.add_value(2, "banana2");
+    db.replace_document(1, doc);
+
+    TEST_EQUAL(doc.get_value(1), "banana1");
+    TEST_EQUAL(doc.get_value(2), "banana2");
+    db.flush();
+
+    doc = db.get_document(1);
+    TEST_EQUAL(doc.get_value(1), "banana1");
+    TEST_EQUAL(doc.get_value(2), "banana2");
+
+    return true;
+}
+
 // Test of new feature: WritableDatabase::replace_document and delete_document
 // can take a unique termname instead of a document id as of Xapian 0.8.2.
 DEFINE_TESTCASE(uniqueterm1, writable) {
@@ -1362,8 +1405,7 @@ DEFINE_TESTCASE(longpositionlist1, writable) {
 
 // Regression test for bug#110: Inconsistent sort order between pages with
 // set_sort_by_value_then_relevance.
-bool test_consistency2()
-{
+DEFINE_TESTCASE(consistency2, writable) {
     Xapian::WritableDatabase db = get_writable_database();
     char buf[2] = "X";
     int i = 0;
@@ -1496,197 +1538,6 @@ DEFINE_TESTCASE(nomoredocids1, writable) {
     return true;
 }
 
-// Test basic spelling correction features.
-DEFINE_TESTCASE(spell1, spelling) {
-    SKIP_TEST_UNLESS_BACKEND("flint");
-
-    Xapian::WritableDatabase db = get_writable_database();
-
-    // Check that the more frequent term is chosen.
-    db.add_spelling("hello");
-    db.add_spelling("cell", 2);
-    TEST_EQUAL(db.get_spelling_suggestion("hell"), "cell");
-    db.flush();
-    Xapian::Database dbr(get_writable_database_as_database());
-    TEST_EQUAL(db.get_spelling_suggestion("hell"), "cell");
-    TEST_EQUAL(dbr.get_spelling_suggestion("hell"), "cell");
-
-    // Check suggestions for single edit errors to "zig".
-    db.add_spelling("zig");
-    // Transpositions:
-    TEST_EQUAL(db.get_spelling_suggestion("izg"), "zig");
-    TEST_EQUAL(db.get_spelling_suggestion("zgi"), "zig");
-    // Substitutions:
-    TEST_EQUAL(db.get_spelling_suggestion("sig"), "zig");
-    TEST_EQUAL(db.get_spelling_suggestion("zog"), "zig");
-    TEST_EQUAL(db.get_spelling_suggestion("zif"), "zig");
-    // Deletions:
-    TEST_EQUAL(db.get_spelling_suggestion("ig"), "zig");
-    TEST_EQUAL(db.get_spelling_suggestion("zg"), "zig");
-    TEST_EQUAL(db.get_spelling_suggestion("zi"), "zig");
-    // Insertions:
-    TEST_EQUAL(db.get_spelling_suggestion("azig"), "zig");
-    TEST_EQUAL(db.get_spelling_suggestion("zaig"), "zig");
-    TEST_EQUAL(db.get_spelling_suggestion("ziag"), "zig");
-    TEST_EQUAL(db.get_spelling_suggestion("ziga"), "zig");
-
-    // Check suggestions for single edit errors to "ch".
-    db.add_spelling("ch");
-    // Transpositions:
-    TEST_EQUAL(db.get_spelling_suggestion("hc"), "ch");
-    // Substitutions - we don't handle these for two character words:
-    TEST_EQUAL(db.get_spelling_suggestion("qh"), "");
-    TEST_EQUAL(db.get_spelling_suggestion("cq"), "");
-    // Deletions would leave a single character, and we don't handle those.
-    TEST_EQUAL(db.get_spelling_suggestion("c"), "");
-    TEST_EQUAL(db.get_spelling_suggestion("h"), "");
-    // Insertions:
-    TEST_EQUAL(db.get_spelling_suggestion("qch"), "ch");
-    TEST_EQUAL(db.get_spelling_suggestion("cqh"), "ch");
-    TEST_EQUAL(db.get_spelling_suggestion("chq"), "ch");
-
-    // Check assorted cases:
-    TEST_EQUAL(db.get_spelling_suggestion("shello"), "hello");
-    TEST_EQUAL(db.get_spelling_suggestion("hellot"), "hello");
-    TEST_EQUAL(db.get_spelling_suggestion("acell"), "cell");
-    TEST_EQUAL(db.get_spelling_suggestion("cella"), "cell");
-    TEST_EQUAL(db.get_spelling_suggestion("acella"), "cell");
-    TEST_EQUAL(db.get_spelling_suggestion("helo"), "hello");
-    TEST_EQUAL(db.get_spelling_suggestion("cll"), "cell");
-    TEST_EQUAL(db.get_spelling_suggestion("helol"), "hello");
-    TEST_EQUAL(db.get_spelling_suggestion("clel"), "cell");
-    TEST_EQUAL(db.get_spelling_suggestion("ecll"), "cell");
-    TEST_EQUAL(db.get_spelling_suggestion("cll"), "cell");
-
-    // Check that edit distance 3 isn't found by default:
-    TEST_EQUAL(db.get_spelling_suggestion("shelolx"), "");
-    TEST_EQUAL(db.get_spelling_suggestion("celling"), "");
-    TEST_EQUAL(db.get_spelling_suggestion("dellin"), "");
-
-    // Check that edit distance 3 is found if specified:
-    TEST_EQUAL(db.get_spelling_suggestion("shelolx", 3), "hello");
-    TEST_EQUAL(db.get_spelling_suggestion("celling", 3), "cell");
-    TEST_EQUAL(db.get_spelling_suggestion("dellin", 3), "cell");
-    return true;
-}
-
-// Test spelling correction for Unicode.
-DEFINE_TESTCASE(spell2, spelling) {
-    SKIP_TEST_UNLESS_BACKEND("flint");
-
-    Xapian::WritableDatabase db = get_writable_database();
-
-    // Check that a UTF-8 sequence counts as a single character.
-    db.add_spelling("h\xc3\xb6hle");
-    db.add_spelling("ascii");
-    TEST_EQUAL(db.get_spelling_suggestion("hohle", 1), "h\xc3\xb6hle");
-    TEST_EQUAL(db.get_spelling_suggestion("hhle", 1), "h\xc3\xb6hle");
-    TEST_EQUAL(db.get_spelling_suggestion("\xf0\xa8\xa8\x8f\xc3\xb6le", 2), "h\xc3\xb6hle");
-    TEST_EQUAL(db.get_spelling_suggestion("hh\xc3\xb6l"), "h\xc3\xb6hle");
-    TEST_EQUAL(db.get_spelling_suggestion("as\xc3\xb6\xc3\xb7i"), "ascii");
-    TEST_EQUAL(db.get_spelling_suggestion("asc\xc3\xb6i\xc3\xb7i"), "ascii");
-    db.flush();
-    Xapian::Database dbr(get_writable_database_as_database());
-    TEST_EQUAL(dbr.get_spelling_suggestion("hohle", 1), "h\xc3\xb6hle");
-    TEST_EQUAL(dbr.get_spelling_suggestion("hhle", 1), "h\xc3\xb6hle");
-    TEST_EQUAL(dbr.get_spelling_suggestion("\xf0\xa8\xa8\x8f\xc3\xb6le", 2), "h\xc3\xb6hle");
-    TEST_EQUAL(dbr.get_spelling_suggestion("hh\xc3\xb6l"), "h\xc3\xb6hle");
-    TEST_EQUAL(dbr.get_spelling_suggestion("as\xc3\xb6\xc3\xb7i"), "ascii");
-    TEST_EQUAL(dbr.get_spelling_suggestion("asc\xc3\xb6i\xc3\xb7i"), "ascii");
-
-    return true;
-}
-
-// Test spelling correction with multi databases
-DEFINE_TESTCASE(spell3, spelling) {
-    SKIP_TEST_UNLESS_BACKEND("flint");
-
-    Xapian::WritableDatabase db1 = get_writable_database();
-    // We can't just call get_writable_database() since it would delete db1
-    // which doesn't work at all under __WIN32__ and will go wrong elsewhere if
-    // changes to db1 are committed.
-    Xapian::WritableDatabase db2 = get_named_writable_database("spell3", "");
-
-    db1.add_spelling("hello");
-    db1.add_spelling("cell", 2);
-    db2.add_spelling("hello", 2);
-    db2.add_spelling("helo");
-
-    Xapian::Database db;
-    db.add_database(db1);
-    db.add_database(db2);
-
-    TEST_EQUAL(db.get_spelling_suggestion("hello"), "");
-    TEST_EQUAL(db.get_spelling_suggestion("hell"), "hello");
-    TEST_EQUAL(db1.get_spelling_suggestion("hell"), "cell");
-    TEST_EQUAL(db2.get_spelling_suggestion("hell"), "hello");
-
-
-    // Test spelling iterator
-    Xapian::TermIterator i(db1.spellings_begin());
-    TEST_EQUAL(*i, "cell");
-    TEST_EQUAL(i.get_termfreq(), 2);
-    ++i;
-    TEST_EQUAL(*i, "hello");
-    TEST_EQUAL(i.get_termfreq(), 1);
-    ++i;
-    TEST(i == db1.spellings_end());
-
-    i = db2.spellings_begin();
-    TEST_EQUAL(*i, "hello");
-    TEST_EQUAL(i.get_termfreq(), 2);
-    ++i;
-    TEST_EQUAL(*i, "helo");
-    TEST_EQUAL(i.get_termfreq(), 1);
-    ++i;
-    TEST(i == db2.spellings_end());
-
-    i = db.spellings_begin();
-    TEST_EQUAL(*i, "cell");
-    TEST_EQUAL(i.get_termfreq(), 2);
-    ++i;
-    TEST_EQUAL(*i, "hello");
-    TEST_EQUAL(i.get_termfreq(), 3);
-    ++i;
-    TEST_EQUAL(*i, "helo");
-    TEST_EQUAL(i.get_termfreq(), 1);
-    ++i;
-    TEST(i == db.spellings_end());
-
-    return true;
-}
-
-// Regression test - check that appending works correctly.
-DEFINE_TESTCASE(spell4, spelling) {
-    SKIP_TEST_UNLESS_BACKEND("flint");
-
-    Xapian::WritableDatabase db = get_writable_database();
-
-    db.add_spelling("check");
-    db.add_spelling("pecks", 2);
-    db.flush();
-    db.add_spelling("becky");
-    db.flush();
-
-    TEST_EQUAL(db.get_spelling_suggestion("jeck", 2), "pecks");
-
-    return true;
-}
-
-// Regression test - used to segfault with some input values.
-DEFINE_TESTCASE(spell5, spelling) {
-    SKIP_TEST_UNLESS_BACKEND("flint");
-
-    Xapian::WritableDatabase db = get_writable_database();
-
-    db.add_spelling("一些");
-    db.flush();
-
-    TEST_EQUAL(db.get_spelling_suggestion("不", strlen("不")), "一些");
-
-    return true;
-}
-
 // Test synonym iterators.
 DEFINE_TESTCASE(synonymitor1, writable) {
     SKIP_TEST_UNLESS_BACKEND("flint");
@@ -1803,97 +1654,6 @@ DEFINE_TESTCASE(synonymitor1, writable) {
     return true;
 }
 
-// Test basic metadata access methods.
-DEFINE_TESTCASE(metadata1, writable) {
-    Xapian::WritableDatabase db = get_writable_database();
-
-    TEST_EQUAL(db.get_metadata("foo"), "");
-    try {
-	db.set_metadata("foo", "bar");
-    } catch (Xapian::UnimplementedError &) {
-	SKIP_TEST("Metadata not supported by this backend");
-    }
-    TEST_EQUAL(db.get_metadata("foo"), "bar");
-    db.set_metadata("foo", "baz");
-    TEST_EQUAL(db.get_doccount(), 0);
-    TEST_EQUAL(db.get_metadata("foo"), "baz");
-    db.set_metadata("foo", "");
-    TEST_EQUAL(db.get_metadata("foo"), "");
-
-    TEST_EQUAL(db.get_doccount(), 0);
-
-    return true;
-}
-
-// Test that metadata gets applied at same time as other changes.
-DEFINE_TESTCASE(metadata2, metadata) {
-    SKIP_TEST_UNLESS_BACKEND("flint");
-    Xapian::WritableDatabase db = get_writable_database();
-    Xapian::Database dbr = get_writable_database_as_database();
-
-    TEST_EQUAL(db.get_metadata("foo"), "");
-    db.set_metadata("foo", "bar");
-    TEST_EQUAL(db.get_metadata("foo"), "bar");
-    TEST_EQUAL(dbr.get_metadata("foo"), "");
-    db.flush();
-    TEST_EQUAL(dbr.get_metadata("foo"), "");
-    dbr.reopen();
-    TEST_EQUAL(db.get_metadata("foo"), "bar");
-    TEST_EQUAL(dbr.get_metadata("foo"), "bar");
-    TEST_EQUAL(dbr.get_doccount(), 0);
-
-    db.add_document(Xapian::Document());
-    db.set_metadata("foo", "baz");
-    TEST_EQUAL(db.get_doccount(), 1);
-    TEST_EQUAL(db.get_metadata("foo"), "baz");
-    db.flush();
-
-    TEST_EQUAL(dbr.get_metadata("foo"), "bar");
-    dbr.reopen();
-    TEST_EQUAL(dbr.get_metadata("foo"), "baz");
-
-    db.set_metadata("foo", "");
-    TEST_EQUAL(db.get_metadata("foo"), "");
-    db.flush();
-    TEST_EQUAL(dbr.get_metadata("foo"), "baz");
-    dbr.reopen();
-    TEST_EQUAL(dbr.get_metadata("foo"), "");
-
-    TEST_EQUAL(db.get_doccount(), 1);
-
-    return true;
-}
-
-// Test the empty metadata keys give an error correctly.
-DEFINE_TESTCASE(metadata3, metadata) {
-    Xapian::WritableDatabase db = get_writable_database();
-
-    TEST_EXCEPTION(Xapian::InvalidArgumentError, db.get_metadata(""));
-    TEST_EXCEPTION(Xapian::InvalidArgumentError, db.set_metadata("", "foo"));
-    TEST_EXCEPTION(Xapian::InvalidArgumentError, db.get_metadata(""));
-
-    return true;
-}
-
-// Regression test for adding a piece of metadata on its own before adding
-// other things.
-DEFINE_TESTCASE(metadata4, metadata) {
-    Xapian::WritableDatabase db = get_writable_database();
-
-    db.set_metadata("foo", "foo");
-    db.flush();
-
-    Xapian::Document doc;
-    doc.add_posting("foo", 1);
-    db.add_document(doc);
-
-    Xapian::Database dbr(get_writable_database_as_database());
-
-    return true;
-}
-
-
-
 // Test that adding a document with a really long term gives an error on
 // add_document() rather than on flush().
 DEFINE_TESTCASE(termtoolong1, writable) {
@@ -2000,15 +1760,68 @@ DEFINE_TESTCASE(postlist7, writable) {
     return true;
 }
 
-/// Regression test of reading after writing but not flushing.
-DEFINE_TESTCASE(writeread1, writable && metadata) {
-    Xapian::WritableDatabase db_w = get_writable_database();
-    db_w.set_metadata("1", "2");
-    string longitem(20000, 'j');
-    db_w.set_metadata("2", longitem);
+DEFINE_TESTCASE(lazytablebug1, writable && flint) {
+    {
+	Xapian::WritableDatabase db = get_named_writable_database("lazytablebug1", string());
 
-    string readitem = db_w.get_metadata("2");
-    TEST_EQUAL(readitem, longitem);
+	Xapian::Document doc;
+	doc.add_term("foo");
+	db.add_document(doc);
+	db.flush();
+
+	string synonym(255, 'x');
+	char buf[] = " iamafish!!!!!!!!!!";
+	for (int i = 33; i < 120; ++i) {
+	    db.add_synonym(buf, synonym);
+	    ++buf[0];
+	}
+
+	db.flush();
+    }
+
+    Xapian::Database db = get_writable_database_as_database();
+    for (Xapian::TermIterator t = db.synonym_keys_begin(); t != db.synonym_keys_end(); ++t) {
+	tout << *t << endl;
+    }
+
+    return true;
+}
+
+/// Regression test for bug #287 for flint.
+DEFINE_TESTCASE(cursordelbug1, flint) {
+    static const int terms[] = { 219, 221, 222, 223, 224, 225, 226 };
+    static const int copies[] = { 74, 116, 199, 21, 45, 155, 189 };
+
+    Xapian::WritableDatabase db;
+    db = get_named_writable_database("cursordelbug1", string());
+
+    for (size_t i = 0; i < sizeof(terms) / sizeof(terms[0]); ++i) {
+	Xapian::Document doc;
+	doc.add_term("XC" + om_tostring(terms[i]));
+	doc.add_term("XTabc");
+	doc.add_term("XAdef");
+	doc.add_term("XRghi");
+	doc.add_term("XYabc");
+	size_t c = copies[i];
+	while (c--) db.add_document(doc);
+    }
+
+    db.flush();
+
+    for (size_t i = 0; i < sizeof(terms) / sizeof(terms[0]); ++i) {
+	db.delete_document("XC" + om_tostring(terms[i]));
+    }
+
+    db.flush();
+
+    string cmd = XAPIAN_BIN_PATH"xapian-check .flint/dbw__cursordelbug1";
+#ifdef __WIN32__
+    cmd += " >nul";
+#else
+    cmd += " >/dev/null";
+#endif
+    if (system(cmd.c_str()) != 0)
+	return false;
 
     return true;
 }
