@@ -1,7 +1,7 @@
 /* testsuite.h: a generic test suite engine
  *
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2005,2006,2007,2008 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,6 +28,7 @@
 
 #include "stringutils.h" // For STRINGIZE().
 
+#include <iomanip>
 #include <map>
 #include <sstream>
 #include <string>
@@ -80,7 +81,7 @@ struct test_desc {
 extern bool verbose;
 
 /// The exception type we were expecting in TEST_EXCEPTION.
-//  Used to detect if such an exception was mishandled by a the
+//  Used to detect if such an exception was mishandled by the
 //  compiler/runtime.
 extern const char * expected_exception;
 
@@ -91,6 +92,9 @@ extern std::ostringstream tout;
 
 /// The test driver.  This class takes care of running the tests.
 class test_driver {
+	/// Write out anything in tout and clear it.
+	void write_and_clear_tout();
+
     public:
 	/** A structure used to report the summary of tests passed
 	 *  and failed.
@@ -104,13 +108,28 @@ class test_driver {
 
 	    /// The number of tests which were skipped
 	    unsigned int skipped;
+
+	    result() : succeeded(0), failed(0), skipped(0) { }
+
+	    result & operator+=(const result & o) {
+		succeeded += o.succeeded;
+		failed += o.failed;
+		skipped += o.skipped;
+		return *this;
+	    }
+
+	    void reset() {
+		succeeded = 0;
+		failed = 0;
+		skipped = 0;
+	    }
 	};
 
 	/** Add a test-specific command line option.
 	 *
 	 *  The recognised option will be described as:
 	 *
-	 *   -<s>=<l>
+	 *   -<s> <l>
 	 *
 	 *  And any value set will be put into arg.
 	 */
@@ -148,7 +167,10 @@ class test_driver {
 	 */
 	static std::string get_srcdir();
 
-	// running total for a test run
+	// Running subtotal for current backend.
+	static result subtotal;
+
+	// Running total for the whole test run.
 	static result total;
 
 	/// Print summary of tests passed, failed, and skipped.
@@ -222,6 +244,9 @@ class test_driver {
 /// Test a condition, without an additional explanation for failure.
 #define TEST(a) TEST_AND_EXPLAIN(a, "")
 
+// Crude wrapper for compatibility with tests backported from 1.1.x.
+#define TEST_REL(A,R,B) TEST((A) R (B))
+
 /// Test for equality of two things.
 #define TEST_EQUAL(a, b) TEST_AND_EXPLAIN(((a) == (b)), \
 	"Expected `"STRINGIZE(a)"' and `"STRINGIZE(b)"' to be equal:" \
